import lib.common.os
from lib.common.evolution.tree_operate import add_clade_name, lookup_by_names, normal_tree_rela_coord, normal_tree_abs_coord, draw_tree, \
    draw_ascii, get_root_by_species, reroot_by_outgroup_clade
import pickle
from api.common.genome.blast import evalue_to_wvalue
import networkx as nx
from Bio import Phylo
import sys
import re
import lib.xuyuxing.evolution.tree_operate_ete3 as to
from api.xuyuxing.file_parser.fileIO import tsv_file_parse
import os
from lib.common.genome.seq_base import read_fasta_big
from config import *
import lib.xuyuxing.base.base_function as bf
import matplotlib.pyplot as plt
import matplotlib

matplotlib.use('Agg')


def PtTree(in_file, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num, MSA_flag):
    if output_dir is None:
        output_dir = "phyout"

    output_tag_file = output_dir.split("/")[-1]

    lib.common.os.mkdir(output_dir, keep=True)
    output_tag = output_dir + "/" + output_tag_file

    # alignment
    aln_file = output_tag + ".aa.aln"
    cmd_string = "sed -i 's/*//g' %s" % (in_file)
    lib.common.os.cmd_run(cmd_string)
    if MSA_flag == "clustalo":
        cmd_string = clustalo_path + " -i %s -o %s -t Protein --force --auto --threads=%d" % (
            in_file, aln_file, cpu_num)
        lib.common.os.cmd_run(cmd_string)
    elif MSA_flag == "clustalw2":
        cmd_string = clustalw_path + " -INFILE=%s -ALIGN -OUTPUT=FASTA -OUTFILE=%s -type=protein" % (
            in_file, aln_file)
        lib.common.os.cmd_run(cmd_string)

    # trimAl
    if trimal_flag:
        used_aln = output_tag + ".trimal.aln"
        used_aln_for_raxml = output_tag_file + ".trimal.aln"
        cmd_string = trimal_path + \
            " -fasta -gappyout -in %s -out %s" % (aln_file, used_aln)
        lib.common.os.cmd_run(cmd_string)
    else:
        used_aln = aln_file
        used_aln_for_raxml = output_tag_file + ".aa.aln"

    # tree builder
    tree_file = output_tag + ".phb"
    tree_file_for_raxml = output_tag_file + ".phb"
    if phylogeneic_tree_builder == 'fasttree':
        cmd_string = fasttree_path + \
            " -wag -gamma -out %s %s" % (tree_file, used_aln)
        lib.common.os.cmd_run(cmd_string)
    elif phylogeneic_tree_builder == 'raxml':
        cmd_string = raxml_path + " -T %d -f a -x 12345 -p 12345 -# 100 -m PROTGAMMAJTT -s %s -n %s" % (
            cpu_num, used_aln_for_raxml, tree_file_for_raxml)
        cwd_string = output_dir
        lib.common.os.cmd_run(cmd_string, cwd_string)

    print("Finished")


def TrueNtTree(in_file, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num, MSA_flag):
    if output_dir is None:
        output_dir = "phyout"

    lib.common.os.mkdir(output_dir)
    output_tag = output_dir + "/" + output_dir

    # alignment
    aln_file = output_tag + ".nt.aln"

    if MSA_flag == "clustalo":
        cmd_string = clustalo_path + \
            " -i %s -o %s --force --auto --threads=%d" % (
                in_file, aln_file, cpu_num)
        lib.common.os.cmd_run(cmd_string)
    elif MSA_flag == "clustalw2":
        cmd_string = clustalw_path + " -INFILE=%s -ALIGN -OUTPUT=FASTA -OUTFILE=%s -type=DNA" % (
            in_file, aln_file)
        lib.common.os.cmd_run(cmd_string)

    # trimAl
    if trimal_flag:
        used_aln = output_tag + ".trimal.aln"
        used_aln_for_raxml = output_dir + ".trimal.aln"
        cmd_string = trimal_path + \
            " -fasta -gappyout -in %s -out %s" % (aln_file, used_aln)
        lib.common.os.cmd_run(cmd_string)
    else:
        used_aln = aln_file
        used_aln_for_raxml = output_dir + ".nt.aln"

    # tree builder
    tree_file = output_tag + ".phb"
    tree_file_for_raxml = output_dir + ".phb"
    if phylogeneic_tree_builder == 'fasttree':
        cmd_string = fasttree_path + \
            " -nt -gtr -gamma -out %s %s" % (tree_file, used_aln)
        lib.common.os.cmd_run(cmd_string)
    elif phylogeneic_tree_builder == 'raxml':
        cmd_string = raxml_path + " -T %d -f a -x 12345 -p 12345 -# 100 -m GTRGAMMA -s %s -n %s" % (
            cpu_num, used_aln_for_raxml, tree_file_for_raxml)
        cwd_string = os.getcwd() + "/" + output_dir
        lib.common.os.cmd_run(cmd_string, cwd_string)

    print("Finished")


def PtNtTree(nucl_file, pt_file, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num):
    if output_dir is None:
        output_dir = "phyout"

    lib.common.os.mkdir(output_dir)
    output_tag = output_dir + "/" + output_dir

    # alignment
    pt_aln_file = output_tag + ".aa.aln"
    cmd_string = "sed -i 's/*//g' %s" % (pt_file)
    lib.common.os.cmd_run(cmd_string)
    cmd_string = clustalo_path + " -i %s -o %s -t Protein --force --auto --threads=%d" % (
        pt_file, pt_aln_file, cpu_num)
    lib.common.os.cmd_run(cmd_string)

    # backtrans
    pt_aln_file = output_tag + ".aa.aln"
    nt_aln_file = output_tag + ".nt.aln"
    cmd_string = treebest_path + \
        " backtrans %s %s > %s" % (pt_aln_file, nucl_file, nt_aln_file)
    lib.common.os.cmd_run(cmd_string)

    # trimAl
    if trimal_flag:
        used_aln = output_tag + ".trimal.aln"
        used_aln_for_raxml = output_dir + ".trimal.aln"
        cmd_string = trimal_path + \
            " -fasta -gappyout -in %s -out %s" % (nt_aln_file, used_aln)
        lib.common.os.cmd_run(cmd_string)
    else:
        used_aln = nt_aln_file
        used_aln_for_raxml = output_dir + ".nt.aln"

    # tree builder
    tree_file = output_tag + ".phb"
    tree_file_for_raxml = output_dir + ".phb"
    if phylogeneic_tree_builder == 'fasttree':
        cmd_string = fasttree_path + \
            " -nt -gtr -gamma -out %s %s" % (tree_file, used_aln)
        lib.common.os.cmd_run(cmd_string)
    elif phylogeneic_tree_builder == 'raxml':
        cmd_string = raxml_path + " -T %d -f a -x 12345 -p 12345 -# 100 -m GTRGAMMA -s %s -n %s" % (
            cpu_num, used_aln_for_raxml, tree_file_for_raxml)
        cwd_string = os.getcwd() + "/" + output_dir
        lib.common.os.cmd_run(cmd_string, cwd_string)

    print("Finished")


def FullFamily(query, database, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num):
    # ToDO
    pass


def PartFamily(query, database, num_top, phylogeneic_tree_builder, trimal_flag, mcl_flag, makeblastdb_flag, output_tag,
               cpu_num):
    from functools import cmp_to_key
    if output_tag is None:
        output_tag = query
    output_tag = output_tag.split("/")[-1]
    output_dir = os.getcwd() + "/" + output_tag
    lib.common.os.mkdir(output_dir, keep=True)
    output_tag = output_dir + "/" + output_tag

    ##

    if makeblastdb_flag is True:
        for files in os.listdir(database):
            if not files.split(".")[-1] == "fasta":
                continue
            cmd_string = "makeblastdb -in %s -dbtype prot" % (
                database + "/" + files)
            lib.common.os.cmd_run(cmd_string)

    query_record = ""
    for files in os.listdir(database):
        if files.split(".")[-1] in ("phr", "pin", "psq"):
            continue
        for record in read_fasta_big(database + "/" + files):
            if record.seqname_short() == query:
                query_record = record
                break
        if query_record != "":
            break
    query_fa = output_tag + ".query.fa"
    with open(query_fa, "w") as f:
        f.write(">" + query_record.seqname_short() + "\n" + query_record.seqs)

    # blast
    blast_result = []
    for files in os.listdir(database):
        if files.split(".")[-1] in ("phr", "pin", "psq"):
            continue
        bls_out = output_tag + "_vs_" + files
        db_file = database + "/" + files
        cmd_string = "blastp -query %s -db %s -out %s -evalue 1e-5 -outfmt 6 -num_threads %d -max_target_seqs %d" % (
            query_fa, db_file, bls_out, cpu_num, num_top)
        lib.common.os.cmd_run(cmd_string)
        blast_result.append(bls_out)

    subject_id = []
    for bls_file in blast_result:
        subject_id.extend(list(tsv_file_parse(bls_file, key_col=2).keys()))
    subject_record = []
    for files in os.listdir(database):
        if files.split(".")[-1] in ("phr", "pin", "psq"):
            continue
        for record in read_fasta_big(database + "/" + files):
            if record.seqname_short() in subject_id:
                subject_record.append(record)
    subject_fa = output_tag + ".blast.subject.fa"
    with open(subject_fa, "w") as f:
        for record in subject_record:
            f.write(">" + record.seqname_short() + "\n" + record.seqs + "\n")

    # mcl cluster
    if mcl_flag is True:
        # all to all blast
        cmd_string = "makeblastdb -in %s -dbtype prot" % subject_fa
        lib.common.os.cmd_run(cmd_string)
        all_to_all_blast = output_tag + ".subject.all_to_all.bls"
        cmd_string = "blastp -query %s -db %s -out %s -evalue 1e-5 -outfmt 6 -num_threads %d" % (
            subject_fa, subject_fa, all_to_all_blast, cpu_num)
        lib.common.os.cmd_run(cmd_string)

        # mcl pre
        tsv_dict = tsv_file_parse(all_to_all_blast, fields="1,2,11")
        mcl_abc_file = output_tag + ".seq.abc"
        with open(mcl_abc_file, "w") as f:
            for a, b, c in [tsv_dict[i] for i in tsv_dict]:
                f.write(a + "\t" + b + "\t" + c + "\n")

        # mcl
        mcl_mci_file = output_tag + ".seq.mci"
        mcl_tab_file = output_tag + ".seq.tab"
        cmd_string = "mcxload -abc %s --stream-mirror --stream-neg-log10 -stream-tf 'ceil(200)' -o %s -write-tab %s" % (
            mcl_abc_file, mcl_mci_file, mcl_tab_file)
        lib.common.os.cmd_run(cmd_string)

        inflation = 1.5
        mcl_out_files = []
        mcl_out_group = {}
        while (inflation <= 6.0):
            mcl_output = "%s.%.1f.mcl.out" % (output_tag, inflation)
            cmd_string = "mcl %s -I %s -use-tab %s -o %s" % (
                mcl_mci_file, inflation, mcl_tab_file, mcl_output)
            lib.common.os.cmd_run(cmd_string)
            mcl_out_files.append(mcl_output)
            mcl_output_dir = tsv_file_parse(mcl_output)
            for group_id in mcl_output_dir:
                if query in mcl_output_dir[group_id]:
                    mcl_out_group[inflation] = mcl_output_dir[group_id]
            inflation = inflation + 0.5

        def cmp_tmp(x, y):
            x_speci_list = list(set([i.split("|")[0]
                                     for i in mcl_out_group[x]]))
            y_speci_list = list(set([i.split("|")[0]
                                     for i in mcl_out_group[y]]))

            if len(x_speci_list) > len(y_speci_list):
                return -1
            elif len(x_speci_list) < len(y_speci_list):
                return 1
            elif len(x_speci_list) == len(y_speci_list):
                if x > y:
                    return -1
                elif y > x:
                    return 1

        best_mcl = sorted(mcl_out_group, key=cmp_to_key(cmp_tmp))[0]
        best_mcl_list = mcl_out_group[best_mcl]

        mcl_cluster_fa = output_tag + ".mcl.fa"
        with open(mcl_cluster_fa, "w") as f:
            for record in subject_record:
                if record.seqname_short() in best_mcl_list:
                    f.write(">" + record.seqname_short() +
                            "\n" + record.seqs + "\n")

        tree_fa = mcl_cluster_fa
    else:
        tree_fa = subject_fa

    PtTree(tree_fa, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num)


def RootGeneTree(gene_tree, species_tree, gene_to_species_map_dict):
    gene_tree = add_clade_name(gene_tree)
    gene_tree_node_dict = lookup_by_names(gene_tree)

    best_root_clade = get_root_by_species(
        gene_tree, species_tree, gene_to_species_map_dict)

    gene_tree_rooted, gene_tree_rooted_node_dict, gene_tree, gene_tree_node_dict = reroot_by_outgroup_clade(gene_tree,
                                                                                                            gene_tree_node_dict,
                                                                                                            best_root_clade.name,
                                                                                                            True)

    gene_tree_rooted_node_dict = lookup_by_names(gene_tree_rooted)

    for i in gene_tree_rooted_node_dict:
        clade = gene_tree_rooted_node_dict[i]
        if not clade.is_terminal():
            clade.name = None

    return gene_tree_rooted



from io import StringIO
from itertools import permutations, combinations
from Bio import Phylo
from Bio.Phylo.BaseTree import Tree, Clade
import re
import copy
import numpy
import sys
import operator
import math
from collections import OrderedDict
from lib.xuyuxing.math.set_operating import jaccord_index
from api.xuyuxing.plot.path_patch import scale_to_give_size, line_segment_plot
from lib.xuyuxing.base.common_command import num_to_chr
import networkx as nx


__author__ = 'Yuxing Xu'

"""
from Bio import Phylo
from lib.common.evolution.tree_operate import add_clade_name

tree_file = "/Users/yuxingxu/Downloads/SpeciesTreeAlignment.fa.raxml.bestTree"
tree = Phylo.read(tree_file, 'newick')
tree = add_clade_name(tree)
"""


# see a tree

def lookup_by_names(tree, order='preorder'):
    """
    order ({'preorder', 'postorder', 'level'}) - Tree traversal order: 'preorder' (default) is depth-first search, 'postorder' is DFS with child nodes preceding parents, and 'level' is breadth-first search.

    order: depth-first search (DFS), preorder from root to first leaf one clade by clade, and second leaf... ; postorder, same search but from leaf to root
    level: breadth-frist search (BFS), from root to leaf one level by level

    see: https://biopython.org/DIST/docs/api/Bio.Phylo.BaseTree.TreeMixin-class.html
    """
    names = OrderedDict()
    for clade in tree.find_clades(order=order):
        if clade.name:
            if clade.name in names:
                raise ValueError("Duplicate key: %s" % clade.name)
            names[clade.name] = clade
    return names


def get_newick_string(tree):
    return tree.format('newick')

# clades relationship


def get_ancestors(child_clade, tree):
    """
    return the list path of parent-child links from the tree root to the clade of choice (root will return null list)
    """
    node_path = tree.get_path(child_clade)
    if len(node_path) == 0:
        return []
    else:
        return [tree.root] + node_path[:-1]


def get_parent(child_clade, tree):
    """
    get most close parent node for child_clade in a tree (root will return null list)
    """
    if child_clade == tree.root:
        return []
    node_path = tree.get_path(child_clade)
    if len(node_path) == 1:
        return tree.root
    else:
        return node_path[-2]


def get_sister(clade, tree):
    """
    get sister clade list
    """

    parent_node = get_parent(clade, tree)
    return [i for i in parent_node.clades if i != clade]


def get_sons(parent_clade):
    """
    get most close son nodes for clade in a tree
    """
    return parent_clade.clades


def get_offspring(parent_clade):
    """
    get all offspring clade for a parent clade
    """
    offspring = []
    for i in parent_clade.find_clades(order='level'):
        if i == parent_clade:
            continue
        offspring.append(i)
    return offspring


def get_leaves(parent_clade, return_name=False):
    if return_name:
        return [leaf.name for leaf in parent_clade.get_terminals()]
    else:
        return [leaf for leaf in parent_clade.get_terminals()]


def get_intranode(parent_clade, return_name=False):
    if return_name:
        return [clade.name for clade in parent_clade.get_nonterminals()]
    else:
        return [clade for clade in parent_clade.get_nonterminals()]


def get_closest_leaf(clade, topology_only=False):
    """Returns node's closest descendant leaf and the distance to
    it.

    :argument False topology_only: If set to True, distance
      between nodes will be referred to the number of nodes
      between them. In other words, topological distance will be
      used instead of branch length distances.

    :return: A tuple containing the closest leaf referred to the
      current node and the distance to it.

    """
    min_dist = None
    min_node = None
    if clade.is_terminal():
        return clade, 0.0
    else:
        for ch in get_sons(clade):
            node, d = get_closest_leaf(ch, topology_only=topology_only)
            if topology_only:
                d += 1.0
            else:
                d += ch.branch_length
            if min_dist is None or d < min_dist:
                min_dist = d
                min_node = node
        return min_node, min_dist


def all_clades_parent(tree):
    parents = {}
    for clade in tree.find_clades(order='level'):
        for child in clade:
            parents[child] = clade
    return parents


def is_offspring_of(A, B, tree):
    """
    A is offspring of B
    """
    return B in get_ancestors(A, tree)


def is_ancestors_of(A, B, tree):
    """
    A is ancestors of B
    """
    return A in get_ancestors(B, tree)


def is_son_of(A, B):
    """
    A is son of B
    """
    return A in get_sons(B)


def is_parent_of(A, B, tree):
    """
    A is parent of B
    """
    return A in get_parent(B, tree)


def is_pass_node(clade, tree):
    if clade.is_terminal():
        return False
    elif clade == tree.root:
        return False
    else:
        clade_son = get_sons(clade)
        if len(clade_son) == 1:
            return True
        else:
            return False


def monophyly(leaf_set, tree, support_threshold=0.8, jaccard_threshold=0.8, exclude_leaf_set=None):
    """
    give a list of gene (gene_set) to check if this gene in a single clade in the tree
    :param tree:
    :param gene_set:
    :param support_threshold:
    :param jaccard_threshold:
    :return:
    """
    tree_old = copy.deepcopy(tree)
    pass_monophyly = []
    for leaf in tree_old.get_terminals():
        if not leaf.name in leaf_set:
            # unroot tree
            tree = copy.deepcopy(tree_old)
            tree = add_clade_name(tree)
            node_dict = lookup_by_names(tree)

            # reroot tree
            tree_new, node_dict_new = reroot_by_outgroup_clade(
                tree, node_dict, leaf.name)
            if not support_threshold is None:
                for clade_name in get_low_support_clade(tree_new, support_threshold):
                    tree_new.collapse(node_dict_new[clade_name])
            node_dict_new = lookup_by_names(tree_new)

            # check monophyly

            for clade_name in node_dict_new:
                clade_leaf_list = [
                    clade_leaf.name for clade_leaf in node_dict_new[clade_name].get_terminals()]
                if exclude_leaf_set:
                    clade_leaf_list_without_exclude = set(
                        clade_leaf_list) - set(exclude_leaf_set)
                else:
                    clade_leaf_list_without_exclude = set(clade_leaf_list)

                if jaccord_index(clade_leaf_list_without_exclude, leaf_set) >= jaccard_threshold:
                    pass_monophyly.append(clade_leaf_list)

    if len(pass_monophyly) > 0:
        min_monophyly_list = sorted(
            pass_monophyly, key=lambda x: jaccord_index(x, leaf_set), reverse=True)[0]
        return True, list(min_monophyly_list)
    else:
        return False, []


def get_MRCA(A, B, tree):
    """
    get most recent common ancestor MRCA
    """
    A_ancest = get_ancestors(A, tree) + [A]
    B_ancest = get_ancestors(B, tree) + [B]

    if len(set(A_ancest) & set(B_ancest)) == 0:
        return tree.root
    else:
        i = 0
        MRCA = None
        while i < len(A_ancest) and i < len(B_ancest) and A_ancest[i] == B_ancest[i]:
            MRCA = A_ancest[i]
            i = i + 1
        return MRCA


def get_MRCA_from_list(clade_list, tree):
    """
    get most recent common ancestor MRCA
    """
    ancest_list = []
    for i in clade_list:
        ancest_list.extend(get_ancestors(i, tree) + [i])

    A_ancest = get_ancestors(clade_list[0], tree) + [clade_list[0]]

    MRCA = A_ancest[0]
    for clade in A_ancest:
        if ancest_list.count(clade) == len(clade_list):
            MRCA = clade
        else:
            break

    return MRCA


def get_distance_to_root(A, tree):
    """
    get distance (sum of branch length) from given clade to root
    """
    ancest_list = get_ancestors(A, tree) + [A]
    distance = 0.0
    for i in ancest_list:
        distance = distance + i.branch_length
    return distance


def get_distance(A, B, tree):
    """
    get distance (sum of branch length) for two clade
    """

    MRCA = get_MRCA(A, B, tree)

    MRCA_to_root = get_distance_to_root(MRCA, tree)
    A_to_root = get_distance_to_root(A, tree)
    B_to_root = get_distance_to_root(B, tree)

    if A_to_root < MRCA_to_root or B_to_root < MRCA_to_root:
        raise ValueError("evolution distance small than ancestor, maybe BUG")

    return (A_to_root - MRCA_to_root) + (B_to_root - MRCA_to_root)


def get_path_to_root(A, tree):
    """
    get path number from A to root
    """
    if A == tree.root:
        return []
    else:
        ancest_list = get_ancestors(A, tree) + [A]
        return ancest_list


def get_path_step_to_root(A, tree):
    """
    get path number from A to root
    """
    if A == tree.root:
        return 0
    else:
        ancest_list = get_ancestors(A, tree) + [A]
        return len(ancest_list)


def get_path_step(A, B, tree):
    """
    get the path number between clades
    """
    MRCA = get_MRCA(A, B, tree)

    MRCA_to_root = get_path_step_to_root(MRCA, tree)
    A_to_root = get_path_step_to_root(A, tree)
    B_to_root = get_path_step_to_root(B, tree)

    if A_to_root < MRCA_to_root or B_to_root < MRCA_to_root:
        raise ValueError("evolution distance small than ancestor, maybe BUG")

    return (A_to_root - MRCA_to_root) + (B_to_root - MRCA_to_root)


# change treeadd_clade_name


def add_clade_name(tree, chr_name=False):
    used_name = []
    for clade in tree.find_clades():
        if clade.name:
            used_name.append(clade.name)

    num = 0
    for clade in tree.find_clades():
        if clade.name is None:
            if chr_name:
                new_id = "Node_%s" % num_to_chr(num, 3)
            else:
                new_id = "N%d" % num
            while new_id in used_name:
                num = num + 1
                if chr_name:
                    new_id = "Node_%s" % num_to_chr(num, 3)
                else:
                    new_id = "N%d" % num
            clade.name = new_id
            used_name.append(new_id)
    return tree


def clade_rename(tree, rename_dict):

    for clade in tree.find_clades():
        if clade.name in rename_dict:
            clade.name = rename_dict[clade.name]

    return tree


def equalCladeTopo(clade1, clade2):
    if clade1.is_terminal() and clade2.is_terminal():
        if clade1.name == clade2.name:
            return True
        else:
            return False
    elif (not clade1.is_terminal()) and (not clade2.is_terminal()):
        if len(clade1.clades) != len(clade2.clades):
            return False
        all_true_flag = True
        for sub_clade1 in clade1.clades:
            true_flag = False
            for sub_clade2 in clade2.clades:
                if equalCladeTopo(sub_clade1, sub_clade2):
                    true_flag = True
            if not true_flag:
                all_true_flag = False
        return all_true_flag
    else:
        return False


def get_low_support_clade(tree, threshold=0.8):
    bad_support = []
    for clade in tree.get_nonterminals():
        if clade.confidence:
            if clade.confidence < threshold:
                bad_support.append(clade.name)
    return bad_support


def collapse_clade_list(tree, name_dict, want_to_collapse_list):
    for j in want_to_collapse_list:
        tree.collapse(name_dict[j])
    return tree


def collapse_low_support(tree, threshold=0.8):
    tree = add_clade_name(tree)
    tree_name_dir = lookup_by_names(tree)
    low_support_clade_list = get_low_support_clade(tree, threshold)
    for j in low_support_clade_list:
        try:
            tree.collapse(tree_name_dir[j])
        except:
            pass
    return tree


def ignore_branch_length(tree, to_None=False):
    """
    remove length to 1.0
    """
    tree = add_clade_name(tree)
    tree_name_dir = lookup_by_names(tree)
    for clade_name in tree_name_dir:
        if to_None:
            tree_name_dir[clade_name].branch_length = None
        else:
            tree_name_dir[clade_name].branch_length = 1.0
    return tree


def remove_given_node(clade, remove_name_list):
    if clade.name in remove_name_list:
        return None
    else:
        if clade.is_terminal():
            return clade
        else:
            son_clade_list = []
            for i in clade.clades:
                filtered_clade = remove_given_node(i, remove_name_list)
                if filtered_clade:
                    son_clade_list.append(filtered_clade)

            if len(son_clade_list) == 0:
                return None
            else:
                clade.clades = son_clade_list
                return clade


def remove_given_node_from_tree(tree, remove_name_list):
    tree_clade = copy.deepcopy(tree.clade)
    clade = remove_given_node(tree_clade, remove_name_list)

    new_tree = Tree(clade, tree.rooted)
    new_tree = remove_pass_node(new_tree)

    return new_tree


def remove_all_node_name(clade):
    clade_now = copy.deepcopy(clade)
    if clade_now.is_terminal():
        return clade_now
    else:
        clade_now.name = None
        new_sub_clade = []
        for i in clade_now.clades:
            new_sub_clade.append(remove_all_node_name(i))
        clade_now.clades = new_sub_clade
        return clade_now


def remove_tree_node_name(tree):
    tree_clade = copy.deepcopy(tree.clade)
    clade = remove_all_node_name(tree_clade)
    new_tree = Tree(clade, tree.rooted)

    return new_tree


def remove_pass_node(tree):
    iter_flag = True
    new_tree = copy.deepcopy(tree)

    while iter_flag:
        tree_tmp = copy.deepcopy(new_tree)
        tree_tmp = add_clade_name(tree_tmp)
        tree_tmp_name_dir = lookup_by_names(tree_tmp, order='level')

        # new_tree_root = tree_name_dir[list(tree_name_dir.keys())[0]]

        # build give son to find parent
        son_to_parent_hash = {}
        for clade_name in tree_tmp_name_dir:
            for son_clade in tree_tmp_name_dir[clade_name].clades:
                if son_clade.name in son_to_parent_hash:
                    raise ValueError("same son for different parent")
                son_to_parent_hash[son_clade.name] = clade_name

        # fund pass node
        pass_node_list = [clade_name for clade_name in tree_tmp_name_dir if
                          is_pass_node(tree_tmp_name_dir[clade_name], tree_tmp)]

        if len(pass_node_list) != 0:
            # for pass node give son to parent
            for pass_node_name in pass_node_list:
                parent_name = son_to_parent_hash[pass_node_name]
                if len(tree_tmp_name_dir[pass_node_name].clades) != 1:
                    raise ValueError("find a error pass node")
                son_name = tree_tmp_name_dir[pass_node_name].clades[0].name

                brother_list = tree_tmp_name_dir[parent_name].clades
                new_brother_list = []
                for i in brother_list:
                    if i.name == pass_node_name:
                        new_brother_list.append(tree_tmp_name_dir[son_name])
                    else:
                        new_brother_list.append(tree_tmp_name_dir[i.name])

                tree_tmp_name_dir[parent_name].clades = new_brother_list

            # build new tree
            root_id = list(tree_tmp_name_dir.keys())[0]
            new_tree = Tree(
                root=tree_tmp_name_dir[root_id], rooted=tree_tmp.rooted)

        else:
            iter_flag = False

    return new_tree


def mid_unrooted_tree(tree):
    """
    root a unrooted tree at midpoint
    """
    tree.root_at_midpoint()
    tree_new = add_clade_name(tree)
    node_dict = lookup_by_names(tree_new)
    return tree_new, node_dict


def reroot_by_outgroup_clade(tree, node_dict, outgroups_clade_name, keep_old=False):
    outgroup_clade = node_dict[outgroups_clade_name]
    outgroup_clade_old = copy.deepcopy(outgroup_clade)

    if keep_old:
        tree_old = copy.deepcopy(tree)

    tree.root_with_outgroup(outgroup_clade)
    tree_new = add_clade_name(tree)

    sis = [i for i in tree_new.clade.clades if i not in outgroup_clade_old.clades][0]

    new_outgroup_clade = copy.deepcopy(outgroup_clade_old)

    if sis.confidence is None:
        sis.confidence = 100

    root_clade = Clade(clades=[sis, new_outgroup_clade], name='root')

    tree_new = Tree(root=root_clade, rooted=True)
    node_dict_new = lookup_by_names(tree_new)

    if keep_old:
        tree_old = add_clade_name(tree_old)
        node_dict_old = lookup_by_names(tree_old)
        return tree_new, node_dict_new, tree_old, node_dict_old
    else:
        return tree_new, node_dict_new


def reroot_by_outgroup_seqid(tree, outgroups_seq_list):
    tree_old = copy.deepcopy(tree)
    for leaf in tree_old.get_terminals():
        if not leaf.name in outgroups_seq_list:
            # unroot tree
            tree = copy.deepcopy(tree_old)
            tree = add_clade_name(tree)
            node_dict = lookup_by_names(tree)

            tree_new, node_dict_new = reroot_by_outgroup_clade(
                tree, node_dict, leaf.name)
            for clade_name in node_dict_new:
                clade_leaf_list = [
                    clade_leaf.name for clade_leaf in node_dict_new[clade_name].get_terminals()]
                if set(clade_leaf_list) == set(outgroups_seq_list):
                    tree_out, node_dict_out = reroot_by_outgroup_clade(
                        tree_new, node_dict_new, clade_name)
                    return tree_out, node_dict_out
    return None


# root a tree rewrite from orthofinder
# need more test

class RootMap(object):
    def __init__(self, setA, setB, GeneToSpecies):
        self.setA = setA
        self.setB = setB
        self.GeneToSpecies = GeneToSpecies

    def GeneMap(self, gene_name):
        sp = self.GeneToSpecies(gene_name)
        if sp in self.setA:
            return True
        elif sp in self.setB:
            return False
        else:
            print(gene_name)
            print(sp)
            raise Exception


def StoreSpeciesSets(t, GeneMap, tag="sp_"):
    tag_up = tag + "up"
    tag_down = tag + "down"
    for clade in t.find_clades(order='postorder'):
        if clade.is_terminal():
            setattr(clade, tag_down, {GeneMap(clade.name)})
        elif clade == t.root:
            continue
        else:
            setattr(clade, tag_down, set.union(
                *[ch.__getattribute__(tag_down) for ch in get_sons(clade)]))

    for clade in t.find_clades(order='preorder'):
        if clade == t.root:
            setattr(clade, tag_up, set())
        else:
            parent = get_parent(clade, t)
            if parent == t.root:
                others = [ch for ch in get_sons(parent) if ch != clade]
                setattr(clade, tag_up, set.union(
                    *[other.__getattribute__(tag_down) for other in others]))
            else:
                others = [ch for ch in get_sons(parent) if ch != clade]
                sp_downs = set.union(
                    *[other.__getattribute__(tag_down) for other in others])
                setattr(clade, tag_up, parent.__getattribute__(
                    tag_up).union(sp_downs))
    # setattr(t, tag_down, set.union(*[ch.__getattribute__(tag_down) for ch in get_sons(t)]))


def OutgroupIngroupSeparationScore(sp_up, sp_down, sett1, sett2, N_recip, n1, n2):
    f_dup = len(sp_up.intersection(sett1)) * len(sp_up.intersection(sett2)) * len(sp_down.intersection(sett1)) * len(
        sp_down.intersection(sett2)) * N_recip
    f_a = len(sp_up.intersection(sett1)) * (n2 - len(sp_up.intersection(sett2))) * (
        n1 - len(sp_down.intersection(sett1))) * len(sp_down.intersection(sett2)) * N_recip
    f_b = (n1 - len(sp_up.intersection(sett1))) * len(sp_up.intersection(sett2)) * len(sp_down.intersection(sett1)) * (
        n2 - len(sp_down.intersection(sett2))) * N_recip
    choice = (f_dup, f_a, f_b)
    #    print(choice)
    return max(choice)


def map_to_method(gene_to_species_map):
    def f(gene_id):
        return gene_to_species_map[gene_id]

    return f


def get_roots_by_species(gene_tree, species_rooted_tree, gene_to_species_map):
    """
    rewrite from orthofinder trees2ologs_of.py GetRoots
    """
    # reads tree
    gene_tree = add_clade_name(gene_tree)
    gene_tree_node_dict = lookup_by_names(gene_tree)
    species_rooted_tree = add_clade_name(species_rooted_tree)
    species_rooted_tree_node_dict = lookup_by_names(species_rooted_tree)
    GeneToSpecies = map_to_method(gene_to_species_map)

    speciesObserved = set([GeneToSpecies(leaf_name)
                           for leaf_name in get_leaves(gene_tree, True)])

    if len(speciesObserved) == 1:
        # arbitrary root if all genes are from the same species
        return [next(n for n in gene_tree.clade)]

    # use species tree to find correct outgroup according to what species are present in the gene tree
    n = species_rooted_tree.clade
    children = get_sons(n)
    leaves = [set(get_leaves(ch, True)) for ch in children]
    # make sure two group have species in gene tree
    have = [len(l.intersection(speciesObserved)) != 0 for l in leaves]
    while sum(have) < 2:
        n = children[have.index(True)]
        children = get_sons(n)
        leaves = [set(get_leaves(ch, True)) for ch in children]
        have = [len(l.intersection(speciesObserved)) != 0 for l in leaves]

    # Get splits to look for
    roots_list = []
    scores_list = []  # the fraction completeness of the two clades
    #    roots_set = set()
    for i in range(0, len(leaves)):
        t1 = leaves[i]
        t2 = set.union(*[l for j, l in enumerate(leaves) if j != i])
        # G - set of species in gene tree
        # First relevant split in species tree is (A,B), such that A \cap G \neq \emptyset and A \cap G \neq \emptyset
        # label all nodes in gene tree according the whether subsets of A, B or both lie below node
        StoreSpeciesSets(gene_tree, GeneToSpecies)  # sets of species
        root_mapper = RootMap(t1, t2, GeneToSpecies)
        sett1 = set(t1)
        sett2 = set(t2)
        nt1 = float(len(t1))
        nt2 = float(len(t2))
        N_recip = 1. / (nt1 * nt1 * nt2 * nt2)
        GeneMap = root_mapper.GeneMap
        # ingroup/outgroup identification
        StoreSpeciesSets(gene_tree, GeneMap, "inout_")
        # find all possible locations in the gene tree at which the root should be

        T = {True, }
        F = {False, }
        TF = set([True, False])
        for m in gene_tree.find_clades(order='postorder'):
            if m.is_terminal():
                if len(m.inout_up) == 1 and m.inout_up != m.inout_down:
                    # this is the unique root
                    return [m]
                    # print(m)
            else:
                if len(m.inout_up) == 1 and len(m.inout_down) == 1 and m.inout_up != m.inout_down:
                    # this is the unique root
                    return [m]
                    # print(m)
                nodes = get_sons(m) if m == gene_tree.root else [
                    m] + get_sons(m)
                clades = [ch.inout_down for ch in nodes] if m == gene_tree.root else (
                    [m.inout_up] + [ch.inout_down for ch in get_sons(m)])
                # do we have the situation A | B or (A,B),S?
                if len(nodes) == 3:
                    if all([len(c) == 1 for c in clades]) and T in clades and F in clades:
                        # unique root
                        if clades.count(T) == 1:
                            return [nodes[clades.index(T)]]
                            # print([nodes[clades.index(T)]])
                        else:
                            return [nodes[clades.index(F)]]
                            # print([nodes[clades.index(F)]])
                    elif T in clades and F in clades:
                        # AB-(A,B) or B-(AB,A)
                        ab = [c == TF for c in clades]
                        i = ab.index(True)
                        roots_list.append(nodes[i])
                        sp_down = nodes[i].sp_down
                        sp_up = nodes[i].sp_up
                        #                        print(m)
                        scores_list.append(
                            OutgroupIngroupSeparationScore(sp_up, sp_down, sett1, sett2, N_recip, nt1, nt2))
                    elif clades.count(TF) >= 2:
                        # (A,A,A)-excluded, (A,A,AB)-ignore as want A to be bigest without including B, (A,AB,AB), (AB,AB,AB)
                        i = 0
                        roots_list.append(nodes[i])
                        sp_down = nodes[i].sp_down
                        sp_up = nodes[i].sp_up
                        #                        print(m)
                        scores_list.append(
                            OutgroupIngroupSeparationScore(sp_up, sp_down, sett1, sett2, N_recip, nt1, nt2))
                elif T in clades and F in clades:
                    roots_list.append(m)
                    scores_list.append(0)  # last choice
    # If we haven't found a unique root then use the scores for completeness of ingroup/outgroup to root
    if len(roots_list) == 0:
        return []  # This shouldn't occur

    return [sorted(zip(scores_list, roots_list), key=lambda x:x[0], reverse=True)[0][1]]


def get_root_by_species(gene_tree, species_rooted_tree, gene_to_species_map):
    roots = get_roots_by_species(
        gene_tree, species_rooted_tree, gene_to_species_map)
    if len(roots) > 0:
        root_dists = [get_closest_leaf(r)[1] for r in roots]
        i, _ = max(enumerate(root_dists), key=operator.itemgetter(1))
        return roots[i]
    else:
        return None  # single species tree


def get_rooted_tree_by_species_tree(unrooted_gene_tree, species_tree, gene_to_species_map_dict):
    unrooted_gene_tree = add_clade_name(unrooted_gene_tree)
    unrooted_gene_tree_node_dict = lookup_by_names(unrooted_gene_tree)

    best_root_clade = get_root_by_species(
        unrooted_gene_tree, species_tree, gene_to_species_map_dict)

    gene_tree_rooted, gene_tree_rooted_node_dict, gene_tree, gene_tree_node_dict = reroot_by_outgroup_clade(unrooted_gene_tree,
                                                                                                            unrooted_gene_tree_node_dict,
                                                                                                            best_root_clade.name,
                                                                                                            True)

    return gene_tree_rooted


def get_root_by_species_for_file(unrooted_tree_file, species_tree_file, gene_to_species_map_dict, output_tree_file):
    species_tree = Phylo.read(species_tree_file, 'newick')
    unrooted_gene_tree = Phylo.read(unrooted_tree_file, 'newick')
    unrooted_gene_tree = add_clade_name(unrooted_gene_tree)
    unrooted_gene_tree_node_dict = lookup_by_names(unrooted_gene_tree)

    best_root_clade = get_root_by_species(
        unrooted_gene_tree, species_tree, gene_to_species_map_dict)

    gene_tree_rooted, gene_tree_rooted_node_dict, gene_tree, gene_tree_node_dict = reroot_by_outgroup_clade(unrooted_gene_tree,
                                                                                                            unrooted_gene_tree_node_dict,
                                                                                                            best_root_clade.name,
                                                                                                            True)

    for i in gene_tree_rooted_node_dict:
        c = gene_tree_rooted_node_dict[i]
        if not c.is_terminal():
            c.name = None

    Phylo.write(gene_tree_rooted, output_tree_file, format='newick')

# for nhx format


def map_node_species_info(rooted_gene_tree, species_tree, GeneToSpecies_dict):
    species_tree_node_dict = lookup_by_names(species_tree)

    for clade in rooted_gene_tree.find_clades(order='preorder'):
        if clade.is_terminal():
            clade.taxon = GeneToSpecies_dict[clade.name]
        else:
            sp_list = list(set([GeneToSpecies_dict[i.name]
                                for i in clade.get_terminals()]))
            sp_clade_list = [species_tree_node_dict[j] for j in sp_list]
            mrca_name = get_MRCA_from_list(sp_clade_list, species_tree).name
            clade.taxon = mrca_name

            num = 0
            sub_clades = {}
            for i in clade.clades:
                sub_clades[num] = i
                num += 1

            duplication = False
            speciation = True
            for a, b in combinations(list(sub_clades.keys()), 2):
                s1 = sub_clades[a]
                s2 = sub_clades[b]

                s1_sp_list = list(set([GeneToSpecies_dict[i.name]
                                       for i in s1.get_terminals()]))
                s2_sp_list = list(set([GeneToSpecies_dict[i.name]
                                       for i in s2.get_terminals()]))

                over_sp = set(s1_sp_list) & set(s2_sp_list)
                if len(over_sp) > 0:
                    duplication = True
                    speciation = False
                    break

            clade.duplication = duplication
            clade.speciation = speciation

    return rooted_gene_tree


def get_nhx_comment(gene_tree):
    for clade in gene_tree.find_clades(order='preorder'):
        if clade.is_terminal():
            clade.comment = "&&NHX:S=%s" % clade.taxon
        else:
            if clade.confidence:
                if isinstance(clade.confidence, float):
                    confidence = min(100, int(clade.confidence * 100))
                else:
                    confidence = clade.confidence
            else:
                confidence = 0
            clade.confidence = None

            clade.comment = "&&NHX:S=%s:D=%s:B=%d" % (
                clade.taxon, 'Y' if clade.duplication else 'N', confidence)

    return gene_tree


def get_nhx_tree_with_taxon_info(gene_tree, species_tree_rooted, GeneToSpecies_dict, output_file):
    gene_tree = add_clade_name(gene_tree)
    species_tree_rooted = add_clade_name(species_tree_rooted)

    gene_tree = map_node_species_info(
        gene_tree, species_tree_rooted, GeneToSpecies_dict)
    nhx_tree = get_nhx_comment(gene_tree)

    with open(output_file, 'w') as f:
        Phylo.write(nhx_tree, f, 'newick')


def nhx_comment_parse(comment_string):
    output_dict = {
        "speciation": 0,
        "duplication": 0,
        "confidence": 0,
        "scientific name": None,
    }

    if comment_string == "&&NHX:None":
        return output_dict
    else:
        info_dict = {}
        for info in comment_string.split(":"):
            info = info.split("=")
            if len(info) > 1:
                info_dict[info[0]] = info[1]

        if 'D' in info_dict and info_dict['D'] == 'Y':
            output_dict["duplication"] = 1
            output_dict["speciation"] = 0
        else:
            output_dict["duplication"] = 0
            output_dict["speciation"] = 1

        if 'B' in info_dict:
            output_dict["confidence"] = int(info_dict['B'])

        if 'S' in info_dict:
            output_dict["scientific name"] = info_dict['S']

        return output_dict


def add_nhx_for_node(clade):
    nhx_dict = nhx_comment_parse(clade.comment)
    clade.nhx_dict = nhx_dict

    if "scientific name" in nhx_dict:
        # clade.species = nhx_dict["scientific name"]
        clade.taxon = nhx_dict["scientific name"]
    if "speciation" in nhx_dict:
        clade.speciation = True if nhx_dict["speciation"] else False
    if "duplication" in nhx_dict:
        clade.duplication = True if nhx_dict["duplication"] else False
    if "confidence" in nhx_dict:
        clade.confidence = True if nhx_dict["confidence"] else False

    return clade


def load_nhx_tree(nhx_file):
    try:
        nhx_tree = Phylo.read(nhx_file, 'newick')
        nhx_tree = add_clade_name(nhx_tree)
        nhx_tree_node_dict = lookup_by_names(nhx_tree)

        # for i in gene_tree_node_dict:
        #     if not gene_tree_node_dict[i].is_terminal():
        #         gene_tree_node_dict[i] = add_nhx_for_node(gene_tree_node_dict[i])

        for i in nhx_tree_node_dict:
            nhx_tree_node_dict[i] = add_nhx_for_node(nhx_tree_node_dict[i])

        return nhx_tree, nhx_tree_node_dict
    except:
        raise ValueError(nhx_file)


def get_top_taxon_clade(taxon_labeled_tree, species_tree, taxonomy, node_type=None):
    sp_tree_dict = lookup_by_names(species_tree)
    sp_list = [i.name for i in sp_tree_dict[taxonomy].get_terminals()]

    candi_clade_list = []
    for clade in taxon_labeled_tree.find_clades(order='preorder'):
        good_type = True

        if clade.taxon == taxonomy:
            good_type = False

            if node_type == 'speciation' and hasattr(clade, 'speciation') and clade.speciation:
                good_type = True
            elif node_type == 'duplication' and hasattr(clade, 'duplication') and clade.duplication:
                good_type = True
            elif node_type is None:
                good_type = True

        if good_type and len(set([l.taxon for l in clade.get_terminals()]) - set(sp_list)) == 0:
            candi_clade_list.append(clade)

    top_clade_list = []
    for i in candi_clade_list:
        top_flag = True
        for j in candi_clade_list:
            if is_offspring_of(i, j, taxon_labeled_tree):
                top_flag = False
        if top_flag:
            top_clade_list.append(i)

    return top_clade_list


# plot tree

# draw function fork from biopython

def draw_ascii(raw_tree, file=None, column_width=80, clade_name=False, clade_confidence=False, rename_dict=None):
    """Draw an ascii-art phylogram of the given tree.

    The printed result looks like::

                                        _________ Orange
                         ______________|
                        |              |______________ Tangerine
          ______________|
         |              |          _________________________ Grapefruit
        _|              |_________|
         |                        |______________ Pummelo
         |
         |__________________________________ Apple


    :Parameters:
        file : file-like object
            File handle opened for writing the output drawing. (Default:
            standard output)
        column_width : int
            Total number of text columns used by the drawing.

    """
    if file is None:
        file = sys.stdout

    tree = copy.deepcopy(raw_tree)

    if rename_dict:
        for clade in tree.find_clades(order='postorder'):
            if clade.name and clade.name in rename_dict:
                clade.name = rename_dict[clade.name]

    taxa = tree.get_terminals()
    # Some constants for the drawing calculations
    max_label_width = max(len(str(taxon)) for taxon in taxa)
    drawing_width = column_width - max_label_width - 1
    drawing_height = 2 * len(taxa) - 1

    def get_col_positions(tree):
        """Create a mapping of each clade to its column position."""
        depths = tree.depths()
        # If there are no branch lengths, assume unit branch lengths
        if not max(depths.values()):
            depths = tree.depths(unit_branch_lengths=True)
        # Potential drawing overflow due to rounding -- 1 char per tree layer
        fudge_margin = int(math.ceil(math.log(len(taxa), 2)))
        cols_per_branch_unit = ((drawing_width - fudge_margin) /
                                float(max(depths.values())))
        return dict((clade, int(blen * cols_per_branch_unit + 1.0))
                    for clade, blen in depths.items())

    def get_row_positions(tree):
        positions = dict((taxon, 2 * idx) for idx, taxon in enumerate(taxa))

        def calc_row(clade):
            for subclade in clade:
                if subclade not in positions:
                    calc_row(subclade)
            positions[clade] = ((positions[clade.clades[0]] +
                                 positions[clade.clades[-1]]) // 2)

        calc_row(tree.root)
        return positions

    col_positions = get_col_positions(tree)
    row_positions = get_row_positions(tree)
    char_matrix = [[' ' for x in range(drawing_width)]
                   for y in range(drawing_height)]

    def draw_clade(clade, startcol):
        thiscol = col_positions[clade]
        thisrow = row_positions[clade]
        # Draw a horizontal line
        for col in range(startcol, thiscol):
            char_matrix[thisrow][col] = '_'
        if clade.clades:
            # Draw a vertical line
            toprow = row_positions[clade.clades[0]]
            botrow = row_positions[clade.clades[-1]]
            for row in range(toprow + 1, botrow + 1):
                char_matrix[row][thiscol] = '|'
            # NB: Short terminal branches need something to stop rstrip()
            if (col_positions[clade.clades[0]] - thiscol) < 2:
                char_matrix[toprow][thiscol] = ','

            # add label for clade draw, by xuyuxing
            clade_string = ""
            if clade_name and clade.name:
                clade_string = clade_string + clade.name + ":"
            if clade_confidence and clade.confidence:
                if len(str(clade.confidence)) <= len("%.2e" % clade.confidence):
                    clade_string = clade_string + str(clade.confidence)
                else:
                    clade_string = clade_string + "%.2e" % clade.confidence
            for i in range(len(clade_string)):
                if len(clade.get_terminals()) % 2:
                    char_matrix[thisrow - 1][thiscol + 1 + i] = clade_string[i]
                else:
                    char_matrix[thisrow][thiscol + 1 + i] = clade_string[i]

            # Draw descendents
            for child in clade:
                draw_clade(child, thiscol + 1)

    draw_clade(tree.root, 0)
    # Print the complete drawing
    for idx, row in enumerate(char_matrix):
        line = ''.join(row).rstrip()
        # Add labels for terminal taxa in the right margin
        if idx % 2 == 0:
            line += ' ' + str(taxa[idx // 2])
        file.write(line + '\n')
    file.write('\n')


def normal_tree_rela_coord(tree):
    """
    add an attr named 'rela_coord' to every clade in the tree. The X is the branch value, and the Y is base on leaf number.
    """
    # get X axis aka branch length from root to clade
    tree = add_clade_name(tree)
    tree_name_dir = lookup_by_names(tree, order='preorder')

    for clade_name in tree_name_dir:
        X_clade = get_distance_to_root(tree_name_dir[clade_name], tree)
        tree_name_dir[clade_name].rela_coord = (X_clade, None)

    # get Y axis
    # first for leaf
    order_leaf_list = [
        clade_name for clade_name in tree_name_dir if tree_name_dir[clade_name].is_terminal()]
    for leaf_name in order_leaf_list:
        X_clade, Y_clade = tree_name_dir[leaf_name].rela_coord
        if (X_clade is None) or (not Y_clade is None):
            raise ValueError("somebady make a rela_coord already")
        Y_clade = float(order_leaf_list.index(leaf_name))
        tree_name_dir[leaf_name].rela_coord = (X_clade, Y_clade)

    # for orther clade, we should cal node which close to leaf
    path_for_target_to_most_far_leaf_dict = {}
    for clade_name in tree_name_dir:
        if clade_name in order_leaf_list:
            continue
        else:
            most_far_path = max([get_path_step(leaf_tmp, tree_name_dir[clade_name], tree) for leaf_tmp in
                                 tree_name_dir[clade_name].get_terminals()])
            if most_far_path not in path_for_target_to_most_far_leaf_dict:
                path_for_target_to_most_far_leaf_dict[most_far_path] = []
            path_for_target_to_most_far_leaf_dict[most_far_path].append(
                clade_name)

    for i in sorted(path_for_target_to_most_far_leaf_dict.keys()):
        for clade_name in path_for_target_to_most_far_leaf_dict[i]:
            X_clade, Y_clade = tree_name_dir[clade_name].rela_coord
            son_Y = [son.rela_coord[1] for son in tree_name_dir[clade_name]]
            Y_clade = (max(son_Y) - min(son_Y)) / 2 + min(son_Y)
            tree_name_dir[clade_name].rela_coord = (X_clade, Y_clade)

    return tree


def normal_tree_abs_coord(tree, figure_size):
    tree_name_dict = lookup_by_names(tree)

    tree_clade_rela_coord = {
        clade_name: tree_name_dict[clade_name].rela_coord for clade_name in tree_name_dict}
    rela_x_list = [x for x, y in tree_clade_rela_coord.values()]
    rela_y_list = [y for x, y in tree_clade_rela_coord.values()]
    rela_fig_size = (min(rela_x_list), max(rela_x_list),
                     min(rela_y_list), max(rela_y_list))

    tree_clade_abs_coord = {
        clade_name: scale_to_give_size(tree_clade_rela_coord[clade_name], figure_size, rela_fig_size) for clade_name in
        tree_clade_rela_coord}

    for clade_name in tree_name_dict:
        tree_name_dict[clade_name].abs_coord = tree_clade_abs_coord[clade_name]

    return tree


def draw_tree(ax, tree):
    tree_name_dict = lookup_by_names(tree, 'level')
    for clade_name in tree_name_dict:
        clade_tmp = tree_name_dict[clade_name]
        son_clade = clade_tmp.clades
        son_y = []
        for son_clade_tmp in son_clade:
            line_segment_plot(ax, (son_clade_tmp.abs_coord[0], son_clade_tmp.abs_coord[1]),
                              (clade_tmp.abs_coord[0], son_clade_tmp.abs_coord[1]))
            son_y.append(son_clade_tmp.abs_coord[1])
        if not clade_tmp.is_terminal():
            line_segment_plot(ax, (clade_tmp.abs_coord[0], min(
                son_y)), (clade_tmp.abs_coord[0], max(son_y)))


# #########################3
#
# def get_speci_id(seq_id):
#     return re.findall(r'^(\d+)\_.*', seq_id)[0]
#
#
# def orthofinder_find_root(gene_tree, speci_tree):
#     """
#     https://www.biorxiv.org/content/biorxiv/early/2019/04/24/466201.full.pdf
#     """
#     pass
#
#
# def get_node_from_target_to_leaf(target_node, leaf):
#     if target_node not in leaf.get_ancestors():
#         raise ValueError("target node is not an ancestor of leaf")
#     output_list = []
#     for i in leaf.get_ancestors():
#         if i == target_node:
#             break
#         output_list.append(i)
#     return output_list
#
#
# def get_node_depth(node, ignore_pass_node):
#     """
#     given a node return the depth of this node, leaf is -1, end node is 0
#     """
#
#     # tmp_dir = {}
#     # for leaves in node.get_leaves():
#     #    tmp_dir[leaves.name]=[tmp_anc for tmp_anc in get_node_from_target_to_leaf(node,leaves) if not pass_node(tmp_anc)]
#
#     if node.is_leaf():
#         return -1
#     else:
#         if ignore_pass_node:
#             return max(
#                 [len([tmp_anc for tmp_anc in get_node_from_target_to_leaf(node, leaves) if not pass_node(tmp_anc)]) for
#                  leaves in node.get_leaves()])
#         else:
#             return max(
#                 [len([tmp_anc for tmp_anc in get_node_from_target_to_leaf(node, leaves)]) for
#                  leaves in node.get_leaves()])
#
#
# if __name__ == '__main__':
#     tree_file = '/lustre/home/xuyuxing/tmp/tree14.tre'
#     tree = Phylo.read(tree_file, "newick")
#     tree = add_clade_name(tree)
#     node_dict = lookup_by_names(tree)
#
#     import pickle
#     import sys
#
#     sys.path.extend(['/lustre/home/xuyuxing/python_project/Genome_work_tools'])
#
#     import argparse
#
#     ###### argument parse
#     parser = argparse.ArgumentParser(
#         prog='PhylTools',
#     )
#
#     subparsers = parser.add_subparsers(title='subcommands', dest="subcommand_name")
#
#     # argparse for get_close_node_info
#     parser_a = subparsers.add_parser('get_close_node_info',
#                                      help='build a protein gene tree from raw sequence',
#                                      description='build a protein gene tree from raw sequence')
#
#     parser_a.add_argument('tree_file', type=str, help='Path of raw protein sequences')
#     parser_a.add_argument('target_speci', type=str, help='Path of raw protein sequences')
#     parser_a.add_argument('taxon_info', type=str, help='Path of raw protein sequences')
#     parser_a.add_argument('taxonomy_dir', type=str, help='Path of raw protein sequences')
#     parser_a.add_argument('taxon_lineage_file', type=str, help='Path of raw protein sequences')
#
#     args = parser.parse_args()
#     args_dict = vars(args)
#
#     # taxon_info = "/lustre/home/xuyuxing/Work/Balanophora/orthofinder/pep2/OrthoFinder/Results_Sep05/WorkingDirectory/tmp/taxon_vs_ortho"
#     # tree_file = "/lustre/home/xuyuxing/Work/Balanophora/orthofinder/pep2/OrthoFinder/Results_Sep05/WorkingDirectory/tmp/OG0000443_tree_id.txt"
#     # taxonomy_dir = "/lustre/home/xuyuxing/Database/genome/info/NCBI/taxonomy"
#     # taxon_lineage_file = "/lustre/home/xuyuxing/Work/Balanophora/orthofinder/pep2/OrthoFinder/Results_Sep05/WorkingDirectory/tmp/lineage.pyb"
#     # target_speci = "15"
#
#     if args_dict["subcommand_name"] == "get_close_node_info":
#
#         from lib.common.fileIO import tsv_file_dict_parse
#
#         tax_info_dict = tsv_file_dict_parse(args.taxon_info, fieldnames=["taxon_id", "speci_ID"], key_col="speci_ID",
#                                             seq=",")
#         tree = Phylo.read(args.tree_file, "newick")
#         tree, node_dict = mid_unrooted_tree(tree)
#
#         leaf_dict = {}
#         leaf_speci_hash = {}
#         for leaf in tree.get_terminals():
#             speci_id = get_speci_id(leaf.name)
#             leaf_speci_hash[leaf.name] = speci_id
#             if speci_id not in leaf_dict:
#                 leaf_dict[speci_id] = []
#             leaf_dict[speci_id].append(leaf.name)
#
#         target_leaf = []
#         for leaf in tree.get_terminals():
#             if leaf_speci_hash[leaf.name] == args.target_speci:
#                 target_leaf.append(leaf.name)
#
#         # from lib.resource.ncbi_taxonomy import build_taxon_database
#         #
#         # tax_record_dict = build_taxon_database(taxonomy_dir)
#         #
#         # lineage_dict = {}
#         # for speci_id in leaf_dict:
#         #     taxon_record = tax_record_dict[tax_info_dict[speci_id]['taxon_id']]
#         #     lineage = taxon_record.get_lineage(tax_record_dict)
#         #     lineage_dict[speci_id] = lineage
#         #
#         # OUT = open(taxon_lineage_file, 'wb')
#         # pickle.dump(lineage_dict, OUT)
#         # OUT.close()
#
#         TEMP = open(args.taxon_lineage_file, 'rb')
#         lineage_dict = pickle.load(TEMP)
#         TEMP.close()
#
#
#         def get_close_node_without_same_speci(tree, child_node, leaf_speci_hash, target_speci):
#             parent_node = get_parent(child_node, tree)
#             close_node_leaf = [leaf.name for leaf in parent_node.get_terminals() if
#                                leaf_speci_hash[leaf.name] != target_speci]
#             if len(close_node_leaf) == 0 and parent_node != tree:
#                 return get_close_node_without_same_speci(tree, parent_node, leaf_speci_hash, target_speci)
#             else:
#                 return parent_node
#
#
#         def common_lineage(lineage_list):
#             if len(lineage_list) == 1:
#                 return lineage_list[0][-1]
#
#             common_taxon = set(lineage_list[0])
#             for i in lineage_list:
#                 common_taxon = set(i) & common_taxon
#
#             a = []
#             for i in lineage_list[0]:
#                 if not i in common_taxon:
#                     return a[-1]
#                 else:
#                     a.append(i)
#
#
#         for leaf_name in leaf_dict[args.target_speci]:
#             close_node = get_close_node_without_same_speci(tree, node_dict[leaf_name], leaf_speci_hash,
#                                                            args.target_speci)
#             close_taxon = common_lineage(
#                 [lineage_dict[get_speci_id(leaf.name)] for leaf in close_node.get_terminals() if
#                  leaf_speci_hash[leaf.name] != args.target_speci])
#             print(args.tree_file, leaf_name, close_node.name, close_taxon[0], close_taxon[1], close_node.confidence)

# for binary tree


class BinaryTreeNode(Clade):
    def __init__(
        self,
        branch_length=None,
        name=None,
        left=None,
        right=None,
        confidence=None,
        color=None,
        width=None,
    ):
        """Define parameters for the BinaryTreeNode Clade."""
        if left is None and right is None:
            clades = None
        else:
            clades = [left, right]
        super(BinaryTreeNode, self).__init__(
            branch_length=branch_length,
            name=name,
            clades=clades,
            confidence=confidence,
            color=color,
            width=width,
        )

        self.left = left
        self.right = right


def normalCalde2BinaryTreeNode(clade):
    if clade.is_terminal():
        return BinaryTreeNode(clade.branch_length, clade.name, None, None, clade.confidence, clade.color, clade.width)
    elif len(clade.clades) == 2:
        left = normalCalde2BinaryTreeNode(clade.clades[0])
        right = normalCalde2BinaryTreeNode(clade.clades[1])
        return BinaryTreeNode(clade.branch_length, clade.name, left, right, clade.confidence, clade.color, clade.width)
    else:
        raise ValueError("clade is not good to be a Binary clade")


def Tree2BinaryTree(tree):
    tree_now = copy.deepcopy(tree)

    tree_clade = tree_now.clade
    BT_clade = normalCalde2BinaryTreeNode(tree_clade)

    new_tree = Tree(BT_clade, tree_now.rooted)

    return new_tree


def allPossibleFBT(N):
    """
    get all possible full binary tree structure, left or right will be treat as difference

    N: all node number, 2 * leaf -1
    return: a list of BinaryTreeNode
    """

    if N == 1:
        return [BinaryTreeNode()]
    res = []

    for i in range(1, N, 2):
        for left in allPossibleFBT(i):
            for right in allPossibleFBT(N-i-1):
                node = BinaryTreeNode(left=left, right=right)
                res.append(node)

    return res


def equalBT(BT1, BT2):
    if BT1.left is None and BT1.right is None and BT2.left is None and BT2.right is None:
        if BT1.name == BT2.name:
            return True
    elif BT1.left is not None and BT1.right is not None and BT2.left is not None and BT2.right is not None:
        if (equalBT(BT1.left, BT2.left) and equalBT(BT1.right, BT2.right)) or (equalBT(BT1.left, BT2.right) and equalBT(BT1.right, BT2.left)):
            return True

    return False


def equalBTtree(BT1, BT2):
    return equalBT(BT1.clade, BT2.clade)


def get_newick(tree):
    f = StringIO()
    Phylo.write(tree, f, 'newick')
    string_tmp = f.getvalue()
    f.close()
    return string_tmp


def merge_same_binary_tree(BT_list):
    edge_list = []
    for i in BT_list:
        for j in BT_list:
            if equalBTtree(i, j):
                edge_list.append((i, j))

    G = nx.Graph()
    G.add_nodes_from(BT_list)
    G.add_edges_from(edge_list)

    output_list = []
    for sub_graph in (G.subgraph(c) for c in nx.connected_components(G)):
        output_list.append(list(sub_graph.nodes))

    return output_list


def all_possible_tree(leaf_list):
    node_num = len(leaf_list) * 2 - 1
    tree_structure_list = [i[0]
                           for i in merge_same_binary_tree(allPossibleFBT(node_num))]

    renamed_tree_list = []
    for tree_structure in tree_structure_list:

        for leaf_order in permutations(leaf_list):
            tree_now = copy.deepcopy(tree_structure)

            num = 0
            for clade in tree_now.find_clades(order='preorder'):
                if clade.is_terminal():
                    # print(num)
                    clade.name = leaf_order[num]
                    num += 1

            renamed_tree_list.append(tree_now)

    renamed_tree_list = [i[0]
                         for i in merge_same_binary_tree(renamed_tree_list)]

    return renamed_tree_list


def gene_loss_model_test(full_model_gene_tree, gene_species_map_dict):
    """
    if we have a gene tree model, but may gene loss will change the topologist of the tree. so test if how many possible case when gene loss

    from io import StringIO
    full_tree_string = "(((Gel_a1,Gel_a2)B1,(Ash_a1,Ash_a2)C1)S1,((Gel_b1,Gel_b2)B2,(Ash_b1,Ash_b2)C2)S2)A,Aco"

    full_model_gene_tree = Phylo.read(StringIO(full_tree_string), 'newick')

    gene_species_map_dict = {
        'Gel_a1': 'Gel',
        'Gel_a2': 'Gel',
        'Gel_b1': 'Gel',
        'Gel_b2': 'Gel',
        'Ash_a1': 'Ash',
        'Ash_a2': 'Ash',
        'Ash_b1': 'Ash',
        'Ash_b2': 'Ash',
    }

    gene not in gene_species_map_dict will not loss

    """

    may_loss_gene_list = gene_species_map_dict.keys()

    keep_case_dict = {}
    num = 0
    for i in range(1, len(may_loss_gene_list)+1):
        for keep_case in combinations(may_loss_gene_list, i):
            keep_case_dict[num] = {
                'keep_gene': keep_case,
                'lost_gene': list(set(may_loss_gene_list) - set(keep_case)),
            }
            num += 1

    # get gene loss tree
    for id_tmp in keep_case_dict:
        lost_gene_id = keep_case_dict[id_tmp]['lost_gene']
        gene_loss_tree = remove_given_node_from_tree(
            full_model_gene_tree, lost_gene_id)
        gene_loss_tree = remove_tree_node_name(gene_loss_tree)
        gene_loss_tree = clade_rename(gene_loss_tree, gene_species_map_dict)
        gene_loss_tree = Tree2BinaryTree(gene_loss_tree)
        keep_case_dict[id_tmp]['BT_tree'] = gene_loss_tree

    # merge
    BT_list = [keep_case_dict[i]['BT_tree'] for i in keep_case_dict]
    uniq_BT_list = [i[0] for i in merge_same_binary_tree(BT_list)]

    uniq_BT_dict = {}
    for uniq_BT in uniq_BT_list:
        uniq_BT_dict[uniq_BT] = []
        for id_tmp in keep_case_dict:
            gene_loss_tree = keep_case_dict[id_tmp]['BT_tree']
            if equalBTtree(uniq_BT, gene_loss_tree):
                uniq_BT_dict[uniq_BT].append(id_tmp)

    return keep_case_dict, uniq_BT_dict


if __name__ == '__main__':
    gene_tree_file = '/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10/Orthologues_Apr11/WorkingDirectory/Trees_ids/OG0000532_tree_id.txt'
    species_rooted_tree_file = '/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10/Orthologues_Apr11/SpeciesTree_rooted.id.txt'

    # reads a tree file
    gene_tree = Phylo.read(gene_tree_file, 'newick')
    gene_tree = add_clade_name(gene_tree)
    gene_tree_node_dict = lookup_by_names(gene_tree)

    species_rooted_tree = Phylo.read(species_rooted_tree_file, 'newick')

    # get all leaf
    leaf_name_list = [leaf.name for leaf in gene_tree.get_terminals()]

    gene_to_species_map = {'1_7601': '1', '8_15565': '8', '1_16489': '1', '8_8464': '8', '9_13535': '9', '6_10585': '6',
                           '2_11400': '2', '5_10489': '5', '4_30192': '4', '5_24848': '5', '6_28832': '6',
                           '2_18137': '2', '4_9871': '4', '5_294': '5', '4_9272': '4', '5_24710': '5', '4_9271': '4',
                           '0_26868': '0', '3_2604': '3', '7_24217': '7', '1_9240': '1', '9_19573': '9', '8_12531': '8',
                           '6_14246': '6', '4_20069': '4', '5_23488': '5', '4_15038': '4', '5_3351': '5', '2_9356': '2',
                           '6_7876': '6', '4_23206': '4', '5_41654': '5', '4_43364': '4', '2_12645': '2',
                           '4_23207': '4', '5_4308': '5', '0_3911': '0', '0_13321': '0', '3_8434': '3', '7_5793': '7'}

    # root gene tree by species info
    best_root_clade = get_root_by_species(
        gene_tree, species_rooted_tree, gene_to_species_map)

    gene_tree_rooted, gene_tree_rooted_node_dict, gene_tree, gene_tree_node_dict = reroot_by_outgroup_clade(gene_tree,
                                                                                                            gene_tree_node_dict,
                                                                                                            best_root_clade.name,
                                                                                                            True)

    # draw a tree
    from io import StringIO

    # normal tree
    tree_string = "(((Orange:0.1,Tangerine:0.2):0.3,(Grapefruit:0.4,Pummelo:0.2):0.1):0.2,Apple:0.6)"
    # tree with comment
    tree_string = "(((Orange:0.1,Tangerine:0.2):0.3[50],(Grapefruit:0.4,Pummelo:0.2):0.1[50]):0.2[60],Apple:0.6)"
    # tree with confidence
    tree_string = "(((Orange:0.1,Tangerine:0.2)50:0.3,(Grapefruit:0.4,Pummelo:0.2)50:0.1)60:0.2,Apple:0.6)"
    tree = Phylo.read(StringIO(tree_string), 'newick')

    draw_ascii(tree)

    # work with nhx tree file
    from lib.common.fileIO import tsv_file_dict_parse

    normal_gene_tree_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD_redo/phylogenomics_old/tree/fasttree/OG0002678/tree.phb"
    species_tree_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD_redo/species.tree.txt"
    rename_map = "/lustre/home/xuyuxing/Work/Gel/orcWGD_redo/phylogenomics_old/tree/fasttree/OG0002678/rename.map"

    gene_tree = Phylo.read(normal_gene_tree_file, 'newick')
    gene_tree = add_clade_name(gene_tree)
    species_tree = Phylo.read(species_tree_file, 'newick')
    species_tree = add_clade_name(species_tree)

    tmp_info = tsv_file_dict_parse(rename_map, fieldnames=[
                                   'new_id', 'old_id', 'speci'], key_col='new_id')
    gene_to_species_map_dict = {i: tmp_info[i]["speci"] for i in tmp_info}

    rooted_gene_tree = get_rooted_tree_by_species_tree(
        gene_tree, species_tree, gene_to_species_map_dict)
    rooted_taxon_gene_tree = map_node_species_info(
        rooted_gene_tree, species_tree, gene_to_species_map_dict)

    nhx_tree = get_nhx_comment(rooted_taxon_gene_tree)

    nhx_file = '/lustre/home/xuyuxing/temp/best.nhx'
    nhx_tree, clade_dict = load_nhx_tree(nhx_file)

    for clade_name in clade_dict:
        clade = clade_dict[clade_name]
        if not clade.is_terminal():
            print(clade.name, clade.nhx_dict['Duplications'])

    # get all possible tree
    leaf_list = ['Gel_a1', 'Gel_a2', 'Ash_a1', 'Ash_a2']
    tree_list = all_possible_tree(leaf_list)

    num = 0
    for i in tree_list:
        print(num)
        num += 1
        Phylo.draw_ascii(i)

    # get all possible tree
    leaf_list = ['Gel', 'Gel', 'Ash', 'Ash']
    tree_list = all_possible_tree(leaf_list)

    num = 0
    for i in tree_list:
        print(num)
        num += 1
        Phylo.draw_ascii(i)

    # gene loss tree
    from io import StringIO
    full_tree_string = "(((Gel_a1,Gel_a2)B1,(Ash_a1,Ash_a2)C1)S1,((Gel_b1,Gel_b2)B2,(Ash_b1,Ash_b2)C2)S2)A,Aco"

    tree = Phylo.read(StringIO(full_tree_string), 'newick')
    draw_ascii(tree, clade_name=True)

    """
                                                     _________________ Gel_a1
                                    ________________|B1:
                                   |                |_________________ Gel_a2
                   ________________|S1:
                  |                |                 _________________ Ash_a1
                  |                |________________|C1:
                  |                                 |_________________ Ash_a2
  ________________|A:
 |                |                                  _________________ Gel_b1
 |                |                 ________________|B2:
 |                |                |                |_________________ Gel_b2
_|                |________________|S2:
 |                                 |                 _________________ Ash_b1
 |                                 |________________|C2:
 |                                                  |_________________ Ash_b2
 |
 |________________ Aco
    
    
    """

    gene_species_map_dict = {
        'Gel_a1': 'Gel',
        'Gel_a2': 'Gel',
        'Gel_b1': 'Gel',
        'Gel_b2': 'Gel',
        'Ash_a1': 'Ash',
        'Ash_a2': 'Ash',
        'Ash_b1': 'Ash',
        'Ash_b2': 'Ash',
    }

    keep_case_dict, uniq_BT_dict = gene_loss_model_test(
        tree, gene_species_map_dict)

    Phylo.draw(tree)

    from Bio.AlignIO import NexusIO

    from Bio import SeqIO

    from Bio.SeqIO import parse

    #

    


import re


class HmmTarget(object):
    """
    q : query
    t : target
    fs : fill sequence
    d : this domain
    hc : hmm coord
    ac : ali coord
    ec : env corrd
    """

    def __init__(self, name=None, accession=None, len=None, fs_evalue=None, fs_score=None, fs_bias=None,
                 description=None, HmmDomain_list=[]):
        self.name = name
        self.accession = accession
        self.description = description
        self.len = int(len)
        self.fs_evalue = float(fs_evalue)
        self.fs_score = float(fs_score)
        self.fs_bias = float(fs_bias)
        self.HmmDomain_list = HmmDomain_list

    def __str__(self):
        return "HmmTarget: %s. Len: %d, have %d HmmDomins\n" % (self.name, self.len, len(self.HmmDomain))


class HmmDomain(object):
    """
    q : query
    t : target
    fs : fill sequence
    d : this domain
    hc : hmm coord
    ac : ali coord
    ec : env corrd
    """

    def __str__(self):
        return "HmmDomins %s, found in %s, %d / %d\n" % (self.q_name, self.target.name, self.d_rank, self.d_total_num)

    def __init__(self, target, q_name=None, q_accesstion=None, q_len=None, d_rank=None, d_total_num=None,
                 d_cevalue=None, d_ievalue=None, d_score=None, d_bias=None, hc_from=None, hc_to=None, ac_from=None,
                 ac_to=None, ec_from=None, ec_to=None, acc=None):
        self.target = target
        self.q_name = q_name
        self.q_accesstion = q_accesstion
        self.q_len = int(q_len)
        self.d_rank = int(d_rank)
        self.d_total_num = int(d_total_num)
        self.d_cevalue = float(d_cevalue)
        self.d_ievalue = float(d_ievalue)
        self.d_score = float(d_score)
        self.d_bias = float(d_bias)
        self.hc_from = int(hc_from)
        self.hc_to = int(hc_to)
        self.ac_from = int(ac_from)
        self.ac_to = int(ac_to)
        self.ec_from = int(ec_from)
        self.ec_to = int(ec_to)
        self.acc = float(acc)


file_name = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/round1/Gel.frag.1.r1.aa_vs_embryophyta_odb10/run_embryophyta_odb10/hmmer_output/16901at3193.out"


def hmmsearch_domtblout_parse(file_name):
    """
    http://eddylab.org/software/hmmer3/3.1b2/Userguide.pdf

    The domain hits table (protein search only)
    In protein search programs, the --domtblout option produces the domain hits table. There is one line for
    each domain. There may be more than one domain per sequence. The domain table has 22 whitespace-delimited
    fields followed by a free text target sequence description, as follows:

    (1) target name: The name of the target sequence or profile.
    (2) target accession: Accession of the target sequence or profile, or ’-’ if none is available.
    (3) tlen: Length of the target sequence or profile, in residues. This (together with the query length) is
    useful for interpreting where the domain coordinates (in subsequent columns) lie in the sequence.
    (4) query name: Name of the query sequence or profile.
    (5) accession: Accession of the target sequence or profile, or ’-’ if none is available.
    (6) qlen: Length of the query sequence or profile, in residues.
    (7) E-value: E-value of the overall sequence/profile comparison (including all domains).
    (8) score: Bit score of the overall sequence/profile comparison (including all domains), inclusive of a
    null2 bias composition correction to the score.
    (9) bias: The biased composition score correction that was applied to the bit score.
    (10) #: This domain’s number (1..ndom).
    (11) of: The total number of domains reported in the sequence, ndom.
    (12) c-Evalue: The “conditional E-value”, a permissive measure of how reliable this particular domain
    may be. The conditional E-value is calculated on a smaller search space than the independent Evalue.
    The conditional E-value uses the number of targets that pass the reporting thresholds. The
    null hypothesis test posed by the conditional E-value is as follows. Suppose that we believe that
    there is already sufficient evidence (from other domains) to identify the set of reported sequences as
    homologs of our query; now, how many additional domains would we expect to find with at least this
    particular domain’s bit score, if the rest of those reported sequences were random nonhomologous
    sequence (i.e. outside the other domain(s) that were sufficient to identified them as homologs in the
    first place)?
    (13) i-Evalue: The “independent E-value”, the E-value that the sequence/profile comparison would have
    received if this were the only domain envelope found in it, excluding any others. This is a stringent
    measure of how reliable this particular domain may be. The independent E-value uses the total
    number of targets in the target database.
    (14) score: The bit score for this domain.
    (15) bias: The biased composition (null2) score correction that was applied to the domain bit score.
    (16) from (hmm coord): The start of the MEA alignment of this domain with respect to the profile,
    numbered 1..N for a profile of N consensus positions.
    (17) to (hmm coord): The end of the MEA alignment of this domain with respect to the profile, numbered
    1..N for a profile of N consensus positions.
    (18) from (ali coord): The start of the MEA alignment of this domain with respect to the sequence,
    numbered 1..L for a sequence of L residues.
    (19) to (ali coord): The end of the MEA alignment of this domain with respect to the sequence, numbered 1..L
    for a sequence of L residues.
    (20) from (env coord): The start of the domain envelope on the sequence, numbered 1..L for a sequence of L residues.
    The envelope defines a subsequence for which their is substantial probability
    mass supporting a homologous domain, whether or not a single discrete alignment can be identified.
    The envelope may extend beyond the endpoints of the MEA alignment, and in fact often does, for
    weakly scoring domains.
    (21) to (env coord): The end of the domain envelope on the sequence, numbered 1..L for a sequence
    of L residues.
    (22) acc: The mean posterior probability of aligned residues in the MEA alignment; a measure of how
    reliable the overall alignment is (from 0 to 1, with 1.00 indicating a completely reliable alignment
    according to the model).
    (23) description of target: The remainder of the line is the target’s description line, as free text.

    q : query
    t : target
    fs : fill sequence
    d : this domain
    hc : hmm coord
    ac : ali coord
    ec : env corrd
    """
    fieldname = ["t_name", "t_accession", "t_len", "q_name", "q_accesstion", "q_len", "fs_evalue", "fs_score",
                 "fs_bias", "d_rank", "d_total_num", "d_cevalue", "d_ievalue", "d_score", "d_bias", "hc_from",
                 "hc_to", "ac_from", "ac_to", "ec_from", "ec_to", "acc", "description"]

    hmm_target_dict = {}
    with open(file_name, "r") as f:
        for each_line in f:
            # read line
            if re.match(r'^#', each_line):
                continue
            each_line = re.sub(r'\n', '', each_line)
            each_line_tuple = each_line.split(maxsplit=22)

            info = {}
            for i in range(len(fieldname)):
                info[fieldname[i]] = each_line_tuple[i]

            # save in object
            if not info['t_name'] in hmm_target_dict:
                hmm_target_dict[info['t_name']] = HmmTarget(name=info['t_name'], accession=info['t_accession'],
                                                            len=info['t_len'], fs_evalue=info['fs_evalue'],
                                                            fs_score=info['fs_score'], fs_bias=info['fs_bias'],
                                                            description=info['description'], HmmDomain_list=[])

            hmm_domin_tmp = HmmDomain(hmm_target_dict[info['t_name']], q_name=info['q_name'],
                                      q_accesstion=info['t_name'], q_len=info['q_len'], d_rank=info['d_rank'],
                                      d_total_num=info['d_total_num'], d_cevalue=info['d_cevalue'],
                                      d_ievalue=info['d_ievalue'], d_score=info['d_score'], d_bias=info['d_bias'],
                                      hc_from=info['hc_from'], hc_to=info['hc_to'], ac_from=info['ac_from'],
                                      ac_to=info['ac_to'], ec_from=info['ec_from'], ec_to=info['ec_to'],
                                      acc=info['acc'])

            hmm_target_dict[info['t_name']].HmmDomain_list.append(hmm_domin_tmp)

    return hmm_target_dict


import os
import re
import pickle
import pandas as pd
from itertools import combinations
from collections import Counter
from interlap import InterLap
from lib.common.os import multiprocess_running
from lib.common.util import printer_list
from lib.common.util import logging_init
from api.xuyuxing.comp_genome.orthofinder import OG_tsv_file_parse
from lib.common.sqlite_command import sqlite_execute
from lib.common.math.bin_and_window import split_sequence_to_bins
from lib.common.math.interval import overlap_between_interval_set, group_by_intervals_with_overlap_threshold
from api.common.genome.blast import outfmt6_read_big
from lib.common.genome.genome_feature2 import get_gf_db_meta_dict, gf_info_retrieval, write_gff_file, read_gff_file, get_mRNA_overlap, convert_dict_structure
from lib.common.genome.seq_base import read_fasta_by_faidx, extract_seq_from_sqlite, write_fasta
from src.xuyuxing.GenomeTools.WPGmapper import load_map_files, extract_all_evidences


def min_cds_start(cds_range_list):
    return min([min(i) for i in cds_range_list])


def get_overlap_ratio(gf1, gf2, similarity_type):
    gf1_cds_interval = [(j.start, j.end)
                        for j in gf1.sub_features[0].sub_features]
    gf2_cds_interval = [(j.start, j.end)
                        for j in gf2.sub_features[0].sub_features]

    overlap_ratio, overlap_length, overlap = overlap_between_interval_set(
        gf1_cds_interval, gf2_cds_interval, similarity_type=similarity_type)

    return overlap_ratio


def higher_in_lower_group(lower_merged_group, higher, cds_interval_dict, similarity_type, threshold):
    grouped_flag = False

    for lower in lower_merged_group:
        jaccord_score = overlap_between_interval_set(
            cds_interval_dict[lower], cds_interval_dict[higher], similarity_type)[0]
        if jaccord_score >= threshold:
            grouped_flag = True
            break

    return grouped_flag


def merge_evidence_by_cds_intervals_identity(ev_id_list, cds_interval_dict, similarity_type, threshold):

    sorted_by_lower_bound = sorted(
        ev_id_list, key=lambda x: min_cds_start(cds_interval_dict[x]))
    merged = Counter()

    num = 0
    for higher in sorted_by_lower_bound:
        if len(merged) == 0:
            merged[num] = [higher]
            num += 1
        else:
            grouped_flag = False
            for g_id in range(len(merged)-1, -1, -1):
                grouped_flag = higher_in_lower_group(
                    merged[g_id], higher, cds_interval_dict, similarity_type, threshold)
                if grouped_flag == True:
                    merged[g_id].append(higher)
                    break

            if not grouped_flag:
                merged[num] = [higher]
                num += 1

    return merged


def evidence_cluster(evidence_dict, similarity_type='jaccord_score', threshold=0.3):
    if len(evidence_dict) > 0:

        # CDS interval list
        cds_interval_dict = {}
        gene_interval_dict = {}
        for i in evidence_dict:
            # if not evidence_dict[i].map_quality_pass:
            #     continue

            gene_interval_dict[i] = (
                evidence_dict[i].start, evidence_dict[i].end)
            cds_interval_dict[i] = [
                (j.start, j.end) for j in evidence_dict[i].sub_features[0].sub_features]

        big_group_dict = group_by_intervals_with_overlap_threshold(
            gene_interval_dict, 0)

        # merge intervals into cluster
        ev_cluster_dict = {}
        num = 0
        for bg_id in big_group_dict:
            # print(bg_id)
            cds_interval_dict_in_group = {
                ev_id: cds_interval_dict[ev_id] for ev_id in big_group_dict[bg_id]['list']}

            ev_cluster_dict_in_group = merge_evidence_by_cds_intervals_identity(
                list(cds_interval_dict_in_group.keys()), cds_interval_dict_in_group, similarity_type, threshold)

            for i in ev_cluster_dict_in_group:
                num += 1
                ev_cluster_dict[num] = ev_cluster_dict_in_group[i]

        return ev_cluster_dict

    else:
        return Counter()


def mark_cluster_supp_score(evidence_dict):

    evidence_cluster_dict = evidence_cluster(
        evidence_dict, similarity_type='jaccord_score', threshold=0.7)

    cluster_support_dict = {}
    for i in evidence_cluster_dict:
        length = len(evidence_cluster_dict[i])
        for j in evidence_cluster_dict[i]:
            cluster_support_dict[j] = length

    output_dict = {}
    for cl_id in evidence_cluster_dict:
        # cl_id_rank = sorted(
        #     evidence_cluster_dict[cl_id], key=lambda x: evidence_dict[x].score_dict['score'], reverse=True)
        cl_id_rank = sorted(
            evidence_cluster_dict[cl_id], key=lambda x: evidence_dict[x].score_dict['bit_score'], reverse=True)
        for i in evidence_cluster_dict[cl_id]:
            gf = evidence_dict[i]
            gf.evidence_indicator["cluster_support"] = cluster_support_dict[i]
            gf.evidence_indicator["cluster_rank"] = cl_id_rank.index(i)
            gf.evidence_indicator["cluster_id"] = cl_id
            output_dict[i] = gf

    return output_dict


def mark_map_quality_for_gf_dict(input_gf_dict, args):
    for ev_id in input_gf_dict:
        # quality control
        coverage_flag = input_gf_dict[ev_id].evidence_indicator["query_coverage"] >= args.min_cover
        aln_aa_len_flag = input_gf_dict[ev_id].evidence_indicator["aln_aa_len"] >= args.min_aa_len
        identity_flag = input_gf_dict[ev_id].evidence_indicator["identity"] >= args.min_identity
        score_flag = input_gf_dict[ev_id].evidence_indicator["score"] >= args.min_score
        cluster_support_flag = input_gf_dict[ev_id].evidence_indicator["cluster_support"] >= args.min_cluster_supp
        OG_support_flag = input_gf_dict[ev_id].evidence_indicator["OG_support"][0] >= args.min_OG_support
        parent_evalue_flag = input_gf_dict[ev_id].parent_blast["evalue"] <= args.evalue
        parent_blast_identity_flag = input_gf_dict[ev_id].parent_blast["identity"] >= args.min_identity
        # support_score_flag = input_gf_dict[ev_id].score_dict["average_supp_score"] > args.min_av_supp_score

        # Special arrow
        special_arrow_flag = input_gf_dict[ev_id].evidence_indicator["query_coverage"] >= 0.8 and input_gf_dict[ev_id].evidence_indicator["identity"] >= 0.9 and input_gf_dict[
            ev_id].parent_blast["identity"] >= 0.9 and input_gf_dict[ev_id].evidence_indicator["aln_aa_len"] >= 200 and input_gf_dict[ev_id].parent_blast["evalue"] <= 1e-30

        # if coverage_flag and aln_aa_len_flag and identity_flag and score_flag and OG_support_flag and support_score_flag and cluster_support_flag:
        if coverage_flag and aln_aa_len_flag and identity_flag and score_flag and OG_support_flag and cluster_support_flag and parent_evalue_flag and parent_blast_identity_flag:
            input_gf_dict[ev_id].map_quality_pass = True
        elif score_flag and special_arrow_flag:
            input_gf_dict[ev_id].map_quality_pass = True
        else:
            input_gf_dict[ev_id].map_quality_pass = False

    return input_gf_dict


def mark_score_rank_for_gf_dict(input_gf_dict):
    gf_score_dict = {}
    for i in input_gf_dict:
        gf = input_gf_dict[i]
        # average_supp_score = gf.score_dict["average_supp_score"]
        # score = gf.score_dict["score"]
        score = gf.score_dict["bit_score"]
        # gf_score_dict[i] = [average_supp_score, score]
        gf_score_dict[i] = [score]

    # get rank score
    # support_rank = sorted(list(gf_score_dict.keys()), key=lambda x:gf_score_dict[x][0], reverse=True)
    self_rank = sorted(list(gf_score_dict.keys()),
                       key=lambda x: gf_score_dict[x][0], reverse=True)

    for i in gf_score_dict:
        gf_score_dict[i].append((self_rank.index(i) + 1))

    output_gf_dict = {}
    for i in gf_score_dict:
        gf = input_gf_dict[i]

        gf.score_dict["self_rank"] = gf_score_dict[i][1]

        output_gf_dict[i] = gf

    return output_gf_dict


def mark_exclusivity_for_gf_dict(input_gf_dict):
    ev_id_list = list(input_gf_dict.keys())

    score_pair_dict = {}
    for i, j in combinations(ev_id_list, 2):
        score_pair_dict[(i, j)] = get_overlap_ratio(
            input_gf_dict[i], input_gf_dict[j], 'shorter_overlap_coverage')

    output_gf_dict = {}
    for ev_id_i in ev_id_list:
        gf = input_gf_dict[ev_id_i]
        exclusivity_list = []
        for ev_id_j in ev_id_list:
            if ev_id_i == ev_id_j:
                continue
            if (ev_id_i, ev_id_j) in score_pair_dict:
                overlap_ratio = score_pair_dict[(ev_id_i, ev_id_j)]
            else:
                overlap_ratio = score_pair_dict[(ev_id_j, ev_id_i)]

            if overlap_ratio > 0.1:
                exclusivity_list.append(ev_id_j)
        gf.exclusivity = exclusivity_list
        gf.exclusivity.append(ev_id_i)
        output_gf_dict[ev_id_i] = gf

    return output_gf_dict

# greedy algorithm


def greedy_get_rep(ev_in_cluster_dict):
    gf_score_rank = sorted(list(ev_in_cluster_dict.keys(
    )), key=lambda x: ev_in_cluster_dict[x].score_dict["self_rank"])

    rep_gf_list = []
    ex_gf_set = set()
    for i in gf_score_rank:
        if i not in ex_gf_set:
            rep_gf_list.append(i)
            ex_gf_set = ex_gf_set | set(ev_in_cluster_dict[i].exclusivity)

    for i in ev_in_cluster_dict:
        if i in rep_gf_list:
            ev_in_cluster_dict[i].is_rep = True
        else:
            ev_in_cluster_dict[i].is_rep = False

    return ev_in_cluster_dict


def rep_gf_cluster(ev_in_cluster_dict):

    ev_in_cluster_dict = mark_score_rank_for_gf_dict(ev_in_cluster_dict)
    ev_in_cluster_dict = mark_exclusivity_for_gf_dict(ev_in_cluster_dict)
    ev_in_cluster_dict = greedy_get_rep(ev_in_cluster_dict)

    return ev_in_cluster_dict


def rep_gf_judge(evidence_dict):
    # give me cleaned evidence_dict
    evidence_dict_cleaned = {
        i: evidence_dict[i] for i in evidence_dict if evidence_dict[i].map_quality_pass}
    output_dict = {i: evidence_dict[i]
                   for i in evidence_dict if not evidence_dict[i].map_quality_pass}

    # for jaccord cluster only use highest ev
    j_cl_dict = {}
    for i in evidence_dict_cleaned:
        gf = evidence_dict_cleaned[i]
        cl_id = gf.evidence_indicator['cluster_id']
        if cl_id not in j_cl_dict:
            j_cl_dict[cl_id] = []
        j_cl_dict[cl_id].append(gf.id)

    cl_best_rep_list = []
    for cl_id in j_cl_dict:
        cl_best_rep = min(
            j_cl_dict[cl_id], key=lambda x: evidence_dict[x].evidence_indicator['cluster_rank'])
        cl_best_rep_list.append(cl_best_rep)

    for i in evidence_dict_cleaned:
        if i not in cl_best_rep_list:
            output_dict[i] = evidence_dict[i]

    evidence_dict_cleaned = {i: evidence_dict[i] for i in cl_best_rep_list}

    #
    evidence_cluster_dict = evidence_cluster(
        evidence_dict_cleaned, similarity_type='shorter_overlap_coverage', threshold=0.1)

    evidence_cluster_dict_hash = {}
    for cl_id in evidence_cluster_dict:
        for gf_id in evidence_cluster_dict[cl_id]:
            evidence_cluster_dict_hash[gf_id] = cl_id

    for cl_id in evidence_cluster_dict:

        ev_in_cluster_dict = {
            ev_id: evidence_dict_cleaned[ev_id] for ev_id in evidence_cluster_dict[cl_id]}
        ev_in_cluster_dict = rep_gf_cluster(ev_in_cluster_dict)

        for ev_id in ev_in_cluster_dict:
            gf = ev_in_cluster_dict[ev_id]
            gf.cl_id = evidence_cluster_dict_hash[gf.id]
            output_dict[ev_id] = gf

    return output_dict


def filter_known_gene(evidence_dict, known_gene_gff, similarity_type='shorter_overlap_coverage', threshold=0.5):

    gff_dict = read_gff_file(known_gene_gff)
    gene_dict, chr_dict = convert_dict_structure(gff_dict)

    chr_interlap_dict = {}
    for contig in chr_dict:
        chr_interlap_dict[contig] = {}
        for strand in chr_dict[contig]:
            chr_interlap_dict[contig][strand] = InterLap()
            ranges = [(chr_dict[contig][strand][i].start, chr_dict[contig][strand]
                       [i].end, chr_dict[contig][strand][i]) for i in chr_dict[contig][strand]]
            if len(ranges) > 0:
                chr_interlap_dict[contig][strand].update(ranges)

    output_gf_dict = {}
    for ev_id in evidence_dict:
        ev_gf = evidence_dict[ev_id]

        overlaped_gene = []
        if ev_gf.chr_id in chr_interlap_dict and ev_gf.strand in chr_interlap_dict[ev_gf.chr_id]:
            for g_s, g_e, gene_gf in chr_interlap_dict[ev_gf.chr_id][ev_gf.strand].find((ev_gf.start, ev_gf.end)):

                overlap_flag = False

                for m1 in ev_gf.sub_features:
                    if overlap_flag:
                        break
                    for m2 in gene_gf.sub_features:
                        if overlap_flag:
                            break
                        if get_mRNA_overlap(m1, m2, similarity_type) > threshold:
                            overlap_flag = True
                            print(ev_id)

                if overlap_flag:
                    overlaped_gene.append(gene_gf.id)

        ev_gf.overlaped_gene_list = overlaped_gene

        if len(overlaped_gene) > 0 and ev_gf and hasattr(ev_gf, 'is_rep') and ev_gf.is_rep:
            ev_gf.is_rep = False

        output_gf_dict[ev_id] = ev_gf

    return output_gf_dict


def get_ortho_support(OG_tsv_file):

    OG_dict = OG_tsv_file_parse(OG_tsv_file)
    seq_support_dict = {}
    for OG_id in OG_dict:
        empty_num = len([i for i in OG_dict[OG_id] if len(
            OG_dict[OG_id][i]) == 1 and OG_dict[OG_id][i][0] == ''])
        total_num = len(OG_dict[OG_id])
        support_num = total_num - empty_num

        for i in OG_dict[OG_id]:
            for j in OG_dict[OG_id][i]:
                if j == '':
                    continue
                seq_support_dict[j] = (support_num, total_num)

    return seq_support_dict


def contig_parser(evidence_dict, args):
    evidence_dict = mark_cluster_supp_score(evidence_dict)
    evidence_dict = mark_map_quality_for_gf_dict(evidence_dict, args)
    evidence_dict = rep_gf_judge(evidence_dict)
    return evidence_dict


def pseudogene_main(args):
    logger_file = args.output_prefix+".log"
    logger = logging_init("PseudoGene", logger_file)

    args_info_string = "Argument list:\n"
    attrs = vars(args)

    for item in attrs.items():
        args_info_string = args_info_string + ("%s: %s\n" % item)

    logger.info(args_info_string)

    logger.info("Read input data from argument list")

    fasta_dict = read_fasta_by_faidx(args.target_genome_fasta)
    contig_dict = {i: fasta_dict[i].len() for i in fasta_dict}

    # load WPGmapper data

    ref_db = pd.read_excel(args.reference_genome_table)
    ref_taxon_id_list = []
    for index in ref_db.index:
        taxon_id = str(ref_db.loc[index]['id'])
        if not pd.isna(taxon_id):
            ref_taxon_id_list.append(taxon_id)

    ref_WPGmapper_dict = load_map_files(
        ref_taxon_id_list, args.WPGmapper_dir)

    # load all evidences
    logger.info("Load all evidences")
    evidence_all_dict_pyb = args.output_prefix+".evidence_all_dict.pyb"
    if os.path.exists(evidence_all_dict_pyb):
        TEMP = open(evidence_all_dict_pyb, 'rb')
        evidence_all_dict = pickle.load(TEMP)
        TEMP.close()
    else:
        seq_support_dict = get_ortho_support(args.OG_tsv_file)
        evidence_meta_dict = extract_all_evidences(
            ref_WPGmapper_dict, evidence_id_list=None, seq_support_dict=seq_support_dict, num_thread=args.threads, log_file=logger_file)

        evidence_all_dict = {}
        for contig in contig_dict:
            evidence_all_dict[contig] = {}
            for strand in ['+', '-']:
                evidence_all_dict[contig][strand] = {}

        for i in evidence_meta_dict:
            gf = evidence_meta_dict[i]
            evidence_all_dict[gf.chr_id][gf.strand][i] = gf

        OUT = open(evidence_all_dict_pyb, 'wb')
        pickle.dump(evidence_all_dict, OUT)
        OUT.close()

    # parse a contig
    logger.info("Parse pseudogene by bins")
    args_list = []
    for contig in contig_dict:
        for strand in ['+', '-']:
            for index, start, end in split_sequence_to_bins(contig_dict[contig], 500000):
                evidence_dict = {i: evidence_all_dict[contig][strand][i] for i in evidence_all_dict[contig][strand]
                                 if evidence_all_dict[contig][strand][i].start <= end and evidence_all_dict[contig][strand][i].end >= start}
                # print(contig,strand,start,end,len(evidence_dict))

                if len(evidence_dict) > 0:
                    args_list.append((evidence_dict, args))

    mp_output = multiprocess_running(
        contig_parser, args_list, args.threads, silence=True, log_file=logger_file)

    huge_evidence_dict = {}
    for i in mp_output:
        evidence_dict = mp_output[i]['output']
        for j in evidence_dict:
            huge_evidence_dict[j] = evidence_dict[j]

    logger.info("filter by known gene")

    huge_evidence_dict = filter_known_gene(
        huge_evidence_dict, args.known_gene_gff, similarity_type='shorter_overlap_coverage', threshold=0.5)

    logger.info("Make report to output")

    all_evidence_list = [huge_evidence_dict[i] for i in huge_evidence_dict]
    map_quality_passed_gf_list = [huge_evidence_dict[i]
                                  for i in huge_evidence_dict if huge_evidence_dict[i].map_quality_pass]
    rep_gf_list = [huge_evidence_dict[i] for i in huge_evidence_dict if hasattr(
        huge_evidence_dict[i], 'is_rep') and huge_evidence_dict[i].is_rep]

    rep_prot_hash = {}
    for gf in rep_gf_list:
        q_id = gf.db_path.split(".")[0]
        if q_id not in rep_prot_hash:
            rep_prot_hash[q_id] = []
        rep_prot_hash[q_id].append(gf.id)

    rep_pt_fasta = args.output_prefix + ".pt.fasta"
    rep_prot_dict = {}
    for q_id in rep_prot_hash:
        gf_id_list = rep_prot_hash[q_id]
        db_name = ref_WPGmapper_dict[q_id]['pep_db']
        gf_seq_dict = extract_seq_from_sqlite(gf_id_list, db_name)
        for i in gf_seq_dict:
            rep_prot_dict[i] = gf_seq_dict[i]

    write_fasta([rep_prot_dict[i] for i in rep_prot_dict],
                rep_pt_fasta, wrap_length=75, upper=True)

    rep_cds_fasta = args.output_prefix + ".cds.fasta"
    rep_prot_dict = {}
    for q_id in rep_prot_hash:
        gf_id_list = rep_prot_hash[q_id]
        db_name = ref_WPGmapper_dict[q_id]['cds_db']
        gf_seq_dict = extract_seq_from_sqlite(gf_id_list, db_name)
        for i in gf_seq_dict:
            rep_prot_dict[i] = gf_seq_dict[i]

    write_fasta([rep_prot_dict[i] for i in rep_prot_dict],
                rep_cds_fasta, wrap_length=75, upper=True)

    all_gff_file = args.output_prefix + ".all.gff3"
    passed_gff_file = args.output_prefix + ".passed.gff3"
    rep_gff_file = args.output_prefix + ".rep.gff3"

    write_gff_file(all_evidence_list, all_gff_file)
    write_gff_file(map_quality_passed_gf_list, passed_gff_file)
    write_gff_file(rep_gf_list, rep_gff_file)

    detail_output_file = args.output_prefix + ".detail.tsv"

    with open(detail_output_file, 'w') as f:
        header = ["gf_id", "chr_id", "strand", "start", "end", "q_id", "q_len", "q_start", "q_end", "q_cover", "q_identity", "q_aln_length", "genewise_score",  "diamond_identity",
                  "diamond_hit_rank", "diamond_bit_score", "diamond_evalue", "OG_support", "r1_cl_id", "r1_cl_support", "map_quality_pass", "r1_cl_rank", "r2_cl_id", "r2_cl_rank", "overlap_with_gene", "is_rep"]
        f.write(printer_list(header)+"\n")

        for gf_id in sorted(huge_evidence_dict, key=lambda x: (huge_evidence_dict[x].chr_id, huge_evidence_dict[x].strand, huge_evidence_dict[x].start)):
            gf = huge_evidence_dict[gf_id]
            gf_id = gf_id
            chr_id = gf.chr_id
            strand = gf.strand
            start = gf.start
            end = gf.end
            q_id = gf.id
            q_len = gf.evidence_indicator['query_length']
            q_start = int(gf.sub_features[0].qualifiers['Target_Start'][0])
            q_end = int(gf.sub_features[0].qualifiers['Target_End'][0])
            q_cover = gf.evidence_indicator['query_coverage']
            q_identity = gf.evidence_indicator['identity']
            q_aln_length = gf.evidence_indicator['aln_aa_len']
            # cds_aln_length = gf.score_dict['cds_aln_length']
            OG_support = "%d/%d" % (
                gf.evidence_indicator['OG_support'][0], gf.evidence_indicator['OG_support'][1])
            genewise_score = gf.score_dict['score']
            diamond_bit_score = gf.score_dict['bit_score']
            diamond_identity = gf.parent_blast["identity"]
            diamond_hit_rank = gf.parent_blast["hit_rank"]
            diamond_evalue = gf.parent_blast["evalue"]
            r1_cl_id = gf.evidence_indicator["cluster_id"]
            r1_cl_support = gf.evidence_indicator['cluster_support']
            map_quality_pass = gf.map_quality_pass
            r1_cl_rank = gf.evidence_indicator["cluster_rank"]
            if hasattr(gf, 'cl_id'):
                r2_cl_id = gf.cl_id
                r2_cl_rank = gf.score_dict['self_rank']
                is_rep = gf.is_rep
            else:
                r2_cl_id = None
                r2_cl_rank = None
                is_rep = False

            if hasattr(gf, 'overlaped_gene_list'):
                overlaped_gene_list = gf.overlaped_gene_list
                overlap_with_gene = printer_list(overlaped_gene_list, ', ')
            else:
                overlap_with_gene = ''

            print_list = [gf_id, chr_id, strand, start, end, q_id, q_len, q_start, q_end, q_cover, q_identity, q_aln_length, genewise_score, diamond_identity,
                          diamond_hit_rank, diamond_bit_score, diamond_evalue, OG_support, r1_cl_id, r1_cl_support, map_quality_pass, r1_cl_rank, r2_cl_id, r2_cl_rank, overlap_with_gene, is_rep]
            f.write(printer_list(print_list)+"\n")


if __name__ == "__main__":

    class abc():
        pass

    args = abc()

    args.WPGmapper_dir = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Ath_WPGmapper"
    args.reference_genome_table = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_ref.xlsx"
    args.target_genome_fasta = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Ath_WPGmapper/T3702N0.genome.fasta"
    args.OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_ortho/protein/OrthoFinder/Results_Oct06/Orthogroups/Orthogroups.tsv"
    args.output_prefix = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/Ath5'
    args.evidence_all_dict_pyb = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/Ath5.evidence_all_dict.pyb'
    args.log_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/Ath5.log'
    args.known_gene_gff = ""

    # A valid evidence alignment region needs to account for at least min_cover of the reference gene
    args.min_cover = 0.5

    args.evalue = 1e-10

    # A valid evidence alignment region needs at least min_aa_len in reference gene
    args.min_aa_len = 50

    # A valid evidence alignment region needs at least min_identity in genewise indentity
    args.min_identity = 0.2

    # A valid evidence alignment region needs at least min_score in genewise score
    args.min_score = 50

    # A valid evidence alignment region needs at least query sequence have min_OG_support orthology from orthofinder
    args.min_OG_support = 2

    # A valid evidence alignment region needs at least min_cluster_supp in jaccord cluster
    args.min_cluster_supp = 2

    args.threads = 80

    # whole genome pseudogene calling
    pseudogene_main(args)

    #
    class abc():
        pass

    args = abc()

    args.WPGmapper_dir = "/lustre/home/xuyuxing/Work/Gel/HGT_WPGmapper/Gel_WPGmapper"
    args.reference_genome_table = "/lustre/home/xuyuxing/Work/Gel/HGT_WPGmapper/fungi_genus.xlsx"
    args.target_genome_fasta = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_WPGmapper/Gel.genome.v2.0.fasta"
    args.OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/HGT/6.orthogrouping/6.1.fungi_ortho/pt_files/OrthoFinder/Results_Jul21/WorkingDirectory/OrthoFinder/Results_Jul22_1/Orthogroups/Orthogroups.tsv"
    args.output_prefix = '/lustre/home/xuyuxing/Work/Gel/HGT_WPGmapper/pseudo/Gel'
    args.evidence_all_dict_pyb = '/lustre/home/xuyuxing/Work/Gel/HGT_WPGmapper/pseudo/Gel.evidence_all_dict.pyb'
    args.log_file = '/lustre/home/xuyuxing/Work/Gel/HGT_WPGmapper/pseudo/Gel.log'

    # A valid evidence alignment region needs to account for at least min_cover of the reference gene
    args.min_cover = 0.5

    args.evalue = 1e-10

    # A valid evidence alignment region needs at least min_aa_len in reference gene
    args.min_aa_len = 50

    # A valid evidence alignment region needs at least min_identity in genewise indentity
    args.min_identity = 0.2

    # A valid evidence alignment region needs at least min_score in genewise score
    args.min_score = 50

    # A valid evidence alignment region needs at least query sequence have min_OG_support orthology from orthofinder
    args.min_OG_support = 2

    # A valid evidence alignment region needs at least min_cluster_supp in jaccord cluster
    args.min_cluster_supp = 2

    args.threads = 80

    # whole genome pseudogene calling
    pseudogene_main(args)

    class abc():
        pass

    args = abc()

    args.WPGmapper_dir = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_WPGmapper"
    args.reference_genome_table = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_ref.xlsx"
    args.target_genome_fasta = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/Gel.genome.v2.0.fasta"
    args.OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_ortho/protein/OrthoFinder/Results_Oct06/Orthogroups/Orthogroups.tsv"
    args.output_prefix = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/redo/Gel'
    args.evidence_all_dict_pyb = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/redo/Gel.evidence_all_dict.pyb'
    args.log_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/redo/Gel.log'
    args.known_gene_gff = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/Gel.genome.gff3"

    # A valid evidence alignment region needs to account for at least min_cover of the reference gene
    args.min_cover = 0.5

    args.evalue = 1e-10

    # A valid evidence alignment region needs at least min_aa_len in reference gene
    args.min_aa_len = 50

    # A valid evidence alignment region needs at least min_identity in genewise indentity
    args.min_identity = 0.2

    # A valid evidence alignment region needs at least min_score in genewise score
    args.min_score = 50

    # A valid evidence alignment region needs at least query sequence have min_OG_support orthology from orthofinder
    args.min_OG_support = 2

    # A valid evidence alignment region needs at least min_cluster_supp in jaccord cluster
    args.min_cluster_supp = 2

    args.threads = 80

    # whole genome pseudogene calling
    pseudogene_main(args)



import os
from config import cutadapt_path
from lib.common.os import get_file_dir, cmd_run, multiprocess_running
import re
import io
import sys
import pysam
from pyfaidx import Fasta
from api.xuyuxing.file_parser.deep_data_parser import split_read_sorted_bam, hit_counter, split_sorted_bam_by_contig
from lib.common.util import logging_init
import array
from lib.common.genome.seq_base import read_fastq_big
from collections import OrderedDict


def clean_fastq_main(tag, fq1, fq2, adapter='illumina'):
    if adapter == 'illumina':
        adapter_string = 'AGATCGGAAGAGC'
    elif adapter == 'mgi_seq':
        adapter_string = 'AAGTCGGAGGCCAAGCGGTCTTAGGAAGACAA'
    else:
        adapter_string = adapter

    script_dir_path = os.path.split(os.path.realpath(__file__))[0]
    btrim_path = script_dir_path + "/../../api/genome/btrim/btrim64-static"
    paired_end_trim_path = script_dir_path + \
        "/../../api/genome/btrim/paired_end_trim.pl"
    work_dir = get_file_dir(fq1)

    # cutadapt
    cutted_fq1 = "%s/%s.cut.1.fq" % (work_dir, tag)
    cutted_fq2 = "%s/%s.cut.2.fq" % (work_dir, tag)

    cmd_string = "%s -a %s -o %s %s" % (cutadapt_path,
                                        adapter_string, cutted_fq1, fq1)
    cmd_run(cmd_string, cwd=work_dir, silence=True)
    cmd_string = "%s -a %s -o %s %s" % (cutadapt_path,
                                        adapter_string, cutted_fq2, fq2)
    cmd_run(cmd_string, cwd=work_dir, silence=True)

    # btrim
    trim1 = "%s/%s.cut.1.fq.trim" % (work_dir, tag)
    trim2 = "%s/%s.cut.2.fq.trim" % (work_dir, tag)
    trim1_sum = "%s/%s.cut.1.fq.trim.sum" % (work_dir, tag)
    trim2_sum = "%s/%s.cut.2.fq.trim.sum" % (work_dir, tag)

    cmd_string = "%s -a 25 -q -t %s -o %s -s %s -P -Q" % (
        btrim_path, cutted_fq1, trim1, trim1_sum)
    cmd_run(cmd_string, cwd=work_dir, silence=True)
    cmd_string = "%s -a 25 -q -t %s -o %s -s %s -P -Q" % (
        btrim_path, cutted_fq2, trim2, trim2_sum)
    cmd_run(cmd_string, cwd=work_dir, silence=True)

    # paired_end_trim
    cmd_string = "%s %s %s %s %s" % (
        paired_end_trim_path, trim1_sum, trim2_sum, trim1, trim2)
    cmd_run(cmd_string, cwd=work_dir, silence=True)
    pe1_file = "%s/%s.cut.1.fq.trim.pe" % (work_dir, tag)
    pe2_file = "%s/%s.cut.2.fq.trim.pe" % (work_dir, tag)
    se1_file = "%s/%s.cut.1.fq.trim.se" % (work_dir, tag)
    se2_file = "%s/%s.cut.2.fq.trim.se" % (work_dir, tag)

    # gzip
    cmd_string = "gzip %s" % pe1_file
    cmd_run(cmd_string, cwd=work_dir, silence=True)
    cmd_string = "gzip %s" % pe2_file
    cmd_run(cmd_string, cwd=work_dir, silence=True)

    # rename and clean
    output1_file = "%s/%s.clean.1.fq.gz" % (work_dir, tag)
    output2_file = "%s/%s.clean.2.fq.gz" % (work_dir, tag)

    cmd_string = "mv %s %s" % (pe1_file+'.gz', output1_file)
    cmd_run(cmd_string, cwd=work_dir, silence=True)
    cmd_string = "mv %s %s" % (pe2_file+'.gz', output2_file)
    cmd_run(cmd_string, cwd=work_dir, silence=True)

    for i in [cutted_fq1, cutted_fq2, trim1, trim2, trim1_sum, trim2_sum, se1_file, se2_file]:
        os.remove(i)


def BamSplit_main(args):
    if args.by_contig:
        if args.keep_head:
            split_sorted_bam_by_contig(
                args.input_bam_file, args.log_file, False)
        else:
            split_sorted_bam_by_contig(args.input_bam_file, args.log_file)
    else:
        split_read_sorted_bam(args.input_bam_file,
                              args.reads_per_file, args.log_file)


def ReadMapStats_main(args):
    """
    class abc(object):
        pass

    args = abc()

    args.log_file = '/lustre/home/xuyuxing/Work/Other/saif/Meth/test/log'
    args.input_bam_file = '/lustre/home/xuyuxing/Work/Other/saif/Meth/test/test.bam'
    args.reads_per_file = 10000
    args.threads = 50
    """


    logger = logging_init(
        "ReadMapStats: stats the reads hits for bam file", args.log_file)

    logger.info("Start: parsing fasta file")

    splited_file_list = split_read_sorted_bam(
        args.input_bam_file, args.reads_per_file, args.log_file)

    args_list = [(i, None) for i in splited_file_list]
    tmp_out = multiprocess_running(
        hit_counter, args_list, args.threads, log_file=args.log_file)

    total_count_stat_dict = {0: 0, 1: 0}
    for i in tmp_out:
        for j in tmp_out[i]['output']:
            if j not in total_count_stat_dict:
                total_count_stat_dict[j] = 0
            total_count_stat_dict[j] = total_count_stat_dict[j] + \
                tmp_out[i]['output'][j]

    total_reads_count = sum(total_count_stat_dict.values())
    uniq_mapping_ratio = total_count_stat_dict[1] / total_reads_count * 100
    total_mapping_ratio = (
        1 - total_count_stat_dict[0] / total_reads_count) * 100

    for i in splited_file_list:
        os.remove(i)

    logger.info("Finished.")

    logger.info("Report: ")
    logger.info("Bam file: %s" % args.input_bam_file)
    logger.info("Reads num: %s" % total_reads_count)
    logger.info("Uniq mapping ratio: %.2f%%" % uniq_mapping_ratio)
    logger.info("total mapping ratio: %.2f%%" % total_mapping_ratio)
    logger.info("reads have hit number: hits\treads")
    for i in range(0, max(list(total_count_stat_dict.keys()))):
        if i not in total_count_stat_dict:
            logger.info("%d\t0" % i)
        else:
            logger.info("%d\t%d" % (i, total_count_stat_dict[i]))

    print("Report: ")
    print("Bam file: %s" % args.input_bam_file)
    print("Reads num: %s" % total_reads_count)
    print("Uniq mapping ratio: %.2f%%" % uniq_mapping_ratio)
    print("total mapping ratio: %.2f%%" % total_mapping_ratio)
    print("reads have hit number: hits\treads")
    for i in range(0, max(list(total_count_stat_dict.keys()))):
        if i not in total_count_stat_dict:
            print("%d\t0" % i)
        else:
            print("%d\t%d" % (i, total_count_stat_dict[i]))


def Fastq2pbbam_main(args):
    raw_fastq = read_fastq_big(args.fastq_file)
    leaf_fastq = read_fastq_big(args.leaf_cut_fastq)
    right_fastq = read_fastq_big(args.right_cut_fastq)

    num = 0
    for read_tmp, in raw_fastq:
        leaf_read_tmp, = next(leaf_fastq)
        right_read_tmp, = next(right_fastq)
        if not (read_tmp.seqname == leaf_read_tmp.seqname == right_read_tmp.seqname):
            raise ValueError("reads name error")

        PU_tag = read_tmp.seqname.split('/')[0]
        ZM_tag = read_tmp.seqname.split('/')[1]
        site_tag = read_tmp.seqname.split('/')[2]
        start, end = site_tag.split('_')

        a = pysam.AlignedSegment()
        a.query_name = read_tmp.seqname
        a.query_sequence = read_tmp.seq
        a.flag = 4
        a.query_qualities = pysam.qualitystring_to_array(read_tmp.quality)
        a.mapping_quality = 255

        # get cx tag
        cx_tag = 0
        if leaf_read_tmp.seq[0].islower():
            cx_tag = cx_tag + 1
        if right_read_tmp.seq[-1].islower():
            cx_tag = cx_tag + 2

        a.tags = (
            ("cx", cx_tag),
            # ("ip", read.tags[1][1]),
            ("ip", array.array('B', [10] * len(a.query_sequence))),
            ("np", 1),
            # ("pw", read.tags[3][1]),
            ("pw", array.array('B', [10] * len(a.query_sequence))),
            ("qe", int(end)),
            ("qs", int(start)),
            ("rq", 0.80),
            ("sn", array.array('f', [5.81927, 11.0514, 6.24621, 10.8449])),
            ("zm", int(ZM_tag)),
            ("RG", 'ad13e653')
        )

        if num == 0:
            header = {'HD': {'VN': '1.5', 'SO': 'unknown', 'pb': '3.0.5'},
                        'RG': [OrderedDict({'ID': 'ad13e653', 'PL': 'PACBIO',
                                            'DS': 'READTYPE=SUBREAD;Ipd:CodecV1=ip;PulseWidth:CodecV1=pw;BINDINGKIT=101-500-400;SEQUENCINGKIT=101-427-500;BASECALLERVERSION=5.0.0;FRAMERATEHZ=100.000000',
                                            'PU': PU_tag, 'PM': 'SEQUEL'})]}

            outf = pysam.AlignmentFile(args.bam_file, "wb", header=header)
            num = num + 1

        outf.write(a)    


from config import esearch_path, efetch_path
from lib.common.os import cmd_run
from lib.common.fileIO import tsv_file_dict_parse
from collections import Counter
import re


def search_sra_data(query_string, output_runinfo_csv):
    """
    query_string = 'Dendrobium'
    output_runinfo_csv = '/lustre/home/xuyuxing/Database/Gel/other/Dca/sra_data/runinfo.csv'
    """

    cmd_string = "%s -db sra -query \"%s\" | %s -format runinfo > %s" % (
        esearch_path, query_string, efetch_path, output_runinfo_csv)
    # print(cmd_string)
    cmd_run(cmd_string)

    return output_runinfo_csv


def runinfo_csv_parser(runinfo_csv_file):
    """
    runinfo_csv_file = '/lustre/home/xuyuxing/Database/Gel/other/Dca/sra_data/runinfo.csv'
    """
    info_dict = tsv_file_dict_parse(runinfo_csv_file, seq=',')

    sra_dict = {}
    for i in info_dict:
        if not info_dict[i]['Run'] == 'Run':
            sra_dict[info_dict[i]['Run']] = info_dict[i]

    return sra_dict


def get_rna_seq_from_sra(keys_word, tmp_runinfo_file):
    """
    keys_word = 'Dendrobium'
    tmp_runinfo_file = '/lustre/home/xuyuxing/Database/Gel/other/Dca/sra_data/runinfo.csv'
    """

    search_sra_data(keys_word, tmp_runinfo_file)
    sra_dict = runinfo_csv_parser(tmp_runinfo_file)

    good_rna_seq_id = []
    for sra_id in sra_dict:
        if not sra_dict[sra_id]['LibraryStrategy'] == 'RNA-Seq':
            continue
        if not sra_dict[sra_id]['LibrarySource'] == 'TRANSCRIPTOMIC':
            continue
        if int(sra_dict[sra_id]['avgLength']) < 50 and int(sra_dict[sra_id]['avgLength']) > 1000:
            continue
        good_rna_seq_id.append(sra_id)

    print("%d in %d is good sra rna_seq" % (len(good_rna_seq_id), len(sra_dict)))

    good_rna_seq = {i: sra_dict[i] for i in good_rna_seq_id}

    return good_rna_seq



"""
args = abc()

args.work_dir = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/test'
args.genome_input_fasta = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/test/short.seq'
args.threads = 56
args.GSS_fq1 = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/test/test.fq1.gz'
args.GSS_fq2 = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/test/test.fq2.gz'
args.related_chl_genome = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/organelle/Lindenbergia_philippensis.plastid.genome.fasta'
args.related_mit_genome = '/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/organelle/close.mitochondrion.genome.fasta'

"""

import os
from lib.common.util import logging_init
from src.xuyuxing.GenomeTools.rRNAFinder import rRNAFinder_main
from src.xuyuxing.GenomeTools.ContigDepth import ContigDepth_main
from lib.common.os import cmd_run, mkdir, copy_file
from lib.common.genome.seq_base import read_fasta_by_faidx
from src.xuyuxing.tools.seqtools import FragmentGenome_main
from src.xuyuxing.tools.taxontools import DiamondTaxonAssign_main
from src.xuyuxing.GenomeTools.ContaminationFilter import ContaminationFilter_main
from config import taxon_nr_db, taxon_dump
from api.common.genome.blast import outfmt6_read
from lib.common.fileIO import tsv_file_dict_parse
import matplotlib.pyplot as plt
import numpy as np


class abc():
    pass


script_dir_path = os.path.split(os.path.realpath(__file__))[0]

ath_chl_aa = script_dir_path+"/organelle/Ath/Ath.chl.aa.fasta"
ath_chl_rRNA = script_dir_path+"/organelle/Ath/Ath.chl.rRNA.fasta"
osa_chl_aa = script_dir_path+"/organelle/Osa/Osa.chl.aa.fasta"
osa_chl_rRNA = script_dir_path+"/organelle/Osa/Osa.chl.rRNA.fasta"

ath_mit_aa = script_dir_path+"/organelle/Ath/Ath.mit.aa.fasta"
ath_mit_rRNA = script_dir_path+"/organelle/Ath/Ath.mit.rRNA.fasta"
osa_mit_aa = script_dir_path+"/organelle/Osa/Osa.mit.aa.fasta"
osa_mit_rRNA = script_dir_path+"/organelle/Osa/Osa.mit.rRNA.fasta"

ath_nucl_rRNA = script_dir_path+"/Ath_rRNA/rRNA.gene.fa"

ref_seq = {
    'chl': {
        'rRNA': {
            'Ath': ath_chl_rRNA,
            'Osa': osa_chl_rRNA,
        },
        'aa': {
            'Ath': ath_chl_aa,
            'Osa': osa_chl_aa,
        },
    },
    'mit': {
        'rRNA': {
            'Ath': ath_mit_rRNA,
            'Osa': osa_mit_rRNA,
        },
        'aa': {
            'Ath': ath_mit_aa,
            'Osa': osa_mit_aa,
        },
    },
    'nuc': {
        'rRNA': {
            'Ath': ath_nucl_rRNA,
        }
    },
}


def contig_filter_plot(contig_filter_file, output_plot_file):
    # read info
    info_dict = tsv_file_dict_parse(contig_filter_file, key_col='contig_id')

    data_dict = {
        'chl': [[], []],
        'mit': [[], []],
        'contamination': [[], []],
        'nuc': [[], []],
    }

    for i in info_dict:
        note_now = info_dict[i]['Note']
        data_dict[note_now][0].append(float(info_dict[i]['GC_ratio']))
        data_dict[note_now][1].append(
            np.log10(float(info_dict[i]['GSS_depth'])))
    #     data_dict[note_now][1].append(float(info_dict[i]['GSS_depth']))

    label_dict = {
        'nuc': ('tab:blue', 'nuclear'),
        'chl': ('tab:green', 'chloroplastidial'),
        'mit': ('tab:orange', 'mitochondrial'),
        'contamination': ('tab:gray', 'contaminated'),
    }

    # plot

    fig, ax = plt.subplots(figsize=(15, 15))

    for note in label_dict:
        scatter = ax.scatter(data_dict[note][0], data_dict[note][1],
                             c=label_dict[note][0], alpha=0.4, label=label_dict[note][1])

    ax.legend(prop={'size': 'xx-large'})
    ax.set_xlabel('GC ratio', fontsize='xx-large')
    ax.set_ylabel('GSS depth', fontsize='xx-large')
    ax.set_title('contig filter', fontsize='xx-large')
    # plt.show()
    fig.savefig(output_plot_file, format='svg',
                facecolor='none', edgecolor='none')


def get_contamination_report(genome_fasta, work_dir, threads=56, max_target_seqs=50):
    mkdir(work_dir, True)

    split_genome_fasta = work_dir + "/split.genome.fasta"
    diamond_output = work_dir + "/nr.diamond.bls"
    taxon_assign_output = work_dir + "/nr.diamond.taxon"
    contamination_report = work_dir + "/contamination_report.txt"

    # FragmentGenome
    args = abc()
    args.genome_file = genome_fasta
    args.output_file = split_genome_fasta
    args.step = 10000
    args.length = 10000
    args.shift_start = 0
    args.Consider_scaffold = True
    args.entropy_threshold = 3

    FragmentGenome_main(args)

    # diamond
    cmd_string = "diamond blastx --query %s --max-target-seqs %d --db %s --evalue 1e-5 --out %s --outfmt 6 qseqid sseqid staxids pident length mismatch gapopen qstart qend sstart send evalue bitscore --threads %s" % (
        split_genome_fasta,
        max_target_seqs,
        taxon_nr_db,
        diamond_output,
        threads
    )
    cmd_run(cmd_string, None, 1, True)

    # DiamondTaxonAssign
    args = abc()
    args.blast_results = diamond_output
    args.tax_dir = taxon_dump
    args.num_threads = threads
    args.output_file = taxon_assign_output

    DiamondTaxonAssign_main(args)

    # ContaminationFilter
    args = abc()
    args.split_genome_fasta = split_genome_fasta
    args.taxon_assign_output = taxon_assign_output
    args.tax_dir = taxon_dump
    args.output_file = contamination_report

    ContaminationFilter_main(args)


def get_best_hsp_score(bls_file):
    bls_dict = outfmt6_read(bls_file)

    max_score_dict = {}

    for q_id in bls_dict:
        query = bls_dict[q_id]
        for hit_now in query.hit:

            if not hit_now.Hit_id in max_score_dict:
                max_score_dict[hit_now.Hit_id] = 0

            for hsp_now in hit_now.hsp:
                if hsp_now.Hsp_bit_score > max_score_dict[hit_now.Hit_id]:
                    max_score_dict[hit_now.Hit_id] = hsp_now.Hsp_bit_score

    return max_score_dict


def ContigFilter_main(args):
    mkdir(args.work_dir, True)
    args.work_dir = os.path.abspath(args.work_dir)
    
    if args.GSS_fq1:
        args.GSS_fq1 = os.path.abspath(args.GSS_fq1)
    if args.GSS_fq2:
        args.GSS_fq2 = os.path.abspath(args.GSS_fq2)
    if args.related_chl_genome:
        args.related_chl_genome = os.path.abspath(args.related_chl_genome)
    if args.related_mit_genome:
        args.related_mit_genome = os.path.abspath(args.related_mit_genome)
    if args.related_nrRNA_seq:
        args.related_nrRNA_seq = os.path.abspath(args.related_nrRNA_seq)
    if args.depth_results:
        args.depth_results = os.path.abspath(args.depth_results)
    if args.contamination_report:
        args.contamination_report = os.path.abspath(args.contamination_report)
    if args.contig_filter_results:
        args.contig_filter_results = os.path.abspath(
            args.contig_filter_results)

    if not args.contig_filter_results:
        copy_file(args.genome_input_fasta,
                  args.work_dir+"/contig_genome.fasta")
        args.genome_input_fasta = os.path.abspath(
            args.work_dir+"/contig_genome.fasta")

        log_file = args.work_dir + "/log"

        logger = logging_init("ContigFilter", log_file)

        # get info
        logger.info("Step 1: get information to filter contigs")

        # 1. get rRNA
        logger.info("Step 1.1: get rRNA in contigs")

        if not args.related_nrRNA_seq:
            args_rRNA = abc()

            args_rRNA.fasta_file = args.genome_input_fasta
            args_rRNA.query_rRNA_seq = None
            args_rRNA.query_rRNA_bed = None
            args_rRNA.log_file = None
            args_rRNA.output_dir = args.work_dir + "/rRNA"

            rRNAFinder_main(args_rRNA)

            args.related_nrRNA_seq = args.work_dir + "/rRNA/rRNA.unit.fa"

        cmd_string = "makeblastdb -in %s -dbtype nucl" % args.genome_input_fasta
        cmd_run(cmd_string, args.work_dir, 1, True, None)

        cmd_string = "blastn -query %s -db %s -outfmt 6 -out vs_rRNA.bls" % (
            args.related_nrRNA_seq, args.genome_input_fasta)
        cmd_run(cmd_string, args.work_dir, 1, True, None)

        logger.info("Step 1.1: Finished")

        # 2. GSS depth
        logger.info("Step 2: get GSS depth")

        logger.info("Step 2.1: get sorted bam file")

        if not args.depth_results:
            hisat2_index = args.genome_input_fasta + ".hisat2"
            cmd_string = "hisat2-build %s %s" % (
                args.genome_input_fasta, hisat2_index)
            cmd_run(cmd_string, args.work_dir, 1, True, None)

            cmd_string = "hisat2 -x %s -p %d -1 %s -2 %s -S GSS.sam" % (
                hisat2_index, args.threads, args.GSS_fq1, args.GSS_fq2)
            # print(cmd_string)
            cmd_run(cmd_string, args.work_dir, 1, True, None)

            cmd_string = "samtools view -bS -@ %d -o GSS.bam GSS.sam" % args.threads
            cmd_run(cmd_string, args.work_dir, 1, True, None)

            cmd_string = "samtools sort -@ %d -o GSS.sorted.bam GSS.bam" % args.threads
            cmd_run(cmd_string, args.work_dir, 1, True, None)

            cmd_string = "samtools index -@ %d GSS.sorted.bam" % args.threads
            cmd_run(cmd_string, args.work_dir, 1, True, None)

            cmd_string = "rm GSS.sam"
            cmd_run(cmd_string, args.work_dir, 1, True, None)

            cmd_string = "rm GSS.bam"
            cmd_run(cmd_string, args.work_dir, 1, True, None)

            logger.info("Step 2.1: Finished")

            logger.info("Step 2.2: get depth report")
            args_GSS = abc()

            args_GSS.bam_file = args.work_dir + "/GSS.sorted.bam"
            args_GSS.ref_genome = args.genome_input_fasta
            args_GSS.output_file = args.work_dir + "/GSS.depth.txt"

            ContigDepth_main(args_GSS)

            args.depth_results = args.work_dir + "/GSS.depth.txt"

        logger.info("Step 2.2: Finished")
        logger.info("Step 2: Finished")

        # 3. GC ratio
        logger.info("Step 3: get GC ratio")

        genome_dict = read_fasta_by_faidx(args.genome_input_fasta)
        gc_dict = {i: genome_dict[i].get_GC_ratio() for i in genome_dict}

        logger.info("Step 3: Finished")

        # 4. blast to reference
        logger.info("Step 4: blast to close plant organelle genome")

        if args.related_chl_genome:
            cmd_string = "blastn -query %s -db %s -outfmt 6 -out vs_chl.bls" % (
                args.related_chl_genome, args.genome_input_fasta)
            cmd_run(cmd_string, args.work_dir, 1, True, None)

            vs_chl_bls_path = args.work_dir + "/vs_chl.bls"
        else:
            vs_chl_bls_path = None

        if args.related_mit_genome:
            cmd_string = "blastn -query %s -db %s -outfmt 6 -out vs_mit.bls" % (
                args.related_mit_genome, args.genome_input_fasta)
            cmd_run(cmd_string, args.work_dir, 1, True, None)

            vs_mit_bls_path = args.work_dir + "/vs_mit.bls"
        else:
            vs_mit_bls_path = None

        logger.info("Step 4: Finished")

        # 5. blastx to Ath or Osa org protein
        logger.info("Step 5: blast to Ath Osa organelle genome")

        ref_org_output_dict = {}
        for org in ref_seq:
            if org == 'nuc':
                continue
            ref_org_output_dict[org] = {}
            for type_tmp in ref_seq[org]:
                ref_org_output_dict[org][type_tmp] = {}
                for sp_id in ref_seq[org][type_tmp]:
                    output_file = args.work_dir + \
                        "/%s.%s.%s.bls" % (org, type_tmp, sp_id)
                    ref_org_output_dict[org][type_tmp][sp_id] = output_file

                    if type_tmp == 'aa':
                        cmd_string = "tblastn -query %s -db %s -outfmt 6 -out %s" % (
                            ref_seq[org][type_tmp][sp_id], args.genome_input_fasta, output_file)
                    elif type_tmp == 'rRNA':
                        cmd_string = "blastn -query %s -db %s -outfmt 6 -out %s" % (
                            ref_seq[org][type_tmp][sp_id], args.genome_input_fasta, output_file)

                    cmd_run(cmd_string, args.work_dir, 1, True, None)
        logger.info("Step 5: Finished")

        # 6. diamond to ncbi, to get contamination report
        logger.info("Step 6: diamond to ncbi, to get contamination report")
        if not args.contamination_report:
            get_contamination_report(
                args.genome_input_fasta, args.work_dir, threads=args.threads, max_target_seqs=50)
            args.contamination_report = args.work_dir + "/contamination_report.txt"

        logger.info("Step 6: Finished")

        # merge info
        logger.info("Step 7: merge all information")

        # get nt_rRNA
        nt_rRNA_bls = args.work_dir + "/vs_rRNA.bls"
        contig_nt_rRNA_score_dict = get_best_hsp_score(nt_rRNA_bls)

        # get chl score
        contig_chl_score_dict = {}
        org = 'chl'
        for type_tmp in ref_org_output_dict[org]:
            for sp_id in ref_org_output_dict[org][type_tmp]:
                tmp_score_dict = get_best_hsp_score(
                    ref_org_output_dict[org][type_tmp][sp_id])
                for contig in tmp_score_dict:
                    if not contig in contig_chl_score_dict:
                        contig_chl_score_dict[contig] = 0
                    if tmp_score_dict[contig] > contig_chl_score_dict[contig]:
                        contig_chl_score_dict[contig] = tmp_score_dict[contig]

        if vs_chl_bls_path:
            tmp_score_dict = get_best_hsp_score(vs_chl_bls_path)
            for contig in tmp_score_dict:
                if not contig in contig_chl_score_dict:
                    contig_chl_score_dict[contig] = 0
                if tmp_score_dict[contig] > contig_chl_score_dict[contig]:
                    contig_chl_score_dict[contig] = tmp_score_dict[contig]

        # get mit score
        contig_mit_score_dict = {}
        org = 'mit'
        for type_tmp in ref_org_output_dict[org]:
            for sp_id in ref_org_output_dict[org][type_tmp]:
                tmp_score_dict = get_best_hsp_score(
                    ref_org_output_dict[org][type_tmp][sp_id])
                for contig in tmp_score_dict:
                    if not contig in contig_mit_score_dict:
                        contig_mit_score_dict[contig] = 0
                    if tmp_score_dict[contig] > contig_mit_score_dict[contig]:
                        contig_mit_score_dict[contig] = tmp_score_dict[contig]

        if vs_mit_bls_path:
            tmp_score_dict = get_best_hsp_score(vs_mit_bls_path)
            for contig in tmp_score_dict:
                if not contig in contig_mit_score_dict:
                    contig_mit_score_dict[contig] = 0
                if tmp_score_dict[contig] > contig_mit_score_dict[contig]:
                    contig_mit_score_dict[contig] = tmp_score_dict[contig]

        # get depth
        info_dict = tsv_file_dict_parse(args.depth_results, key_col='contig')
        depth_dict = {c_id: float(info_dict[c_id]['avg_depth'])
                      for c_id in info_dict}

        # gc
        # gc_dict

        # read contamination_report.txt
        info_dict = tsv_file_dict_parse(
            args.contamination_report, key_col='contig_id')

        contamination_dict = {}
        for c_id in info_dict:
            total_num = sum([int(info_dict[c_id][i])
                             for i in info_dict[c_id] if i != 'contig_id'])
            plant_num = sum([int(info_dict[c_id][i])
                             for i in info_dict[c_id] if i == 'Viridiplantae'])
            other_num = sum([int(info_dict[c_id][i]) for i in info_dict[c_id]
                             if i != 'contig_id' and i != 'Viridiplantae' and i != 'No_hit'])
            contamination_dict[c_id] = other_num > plant_num

        # make report
        args.contig_filter_results = args.work_dir + "/ContigFilter.txt"

        with open(args.contig_filter_results, 'w') as f:
            title_list = ["contig_id", "contig_length", "GC_ratio", "GSS_depth", "Chl_score",
                          "Mit_score", "ntrRNA_score", "NCBI_hit", "Note"]
            title_string = "\t".join(title_list)
            f.write(title_string+"\n")

            for c_id in list(genome_dict.keys()):
                c_len = genome_dict[c_id].len()
                gc_value = gc_dict[c_id] if c_id in gc_dict else 0.0
                depth_value = depth_dict[c_id] if c_id in depth_dict else 0.0
                chl_s = contig_chl_score_dict[c_id] if c_id in contig_chl_score_dict else 0.0
                mit_s = contig_mit_score_dict[c_id] if c_id in contig_mit_score_dict else 0.0
                nt_s = contig_nt_rRNA_score_dict[c_id] if c_id in contig_nt_rRNA_score_dict else 0.0
                cont_b = contamination_dict[c_id] if c_id in contamination_dict else None

                # Note
                if chl_s > 5000.0 and depth_value > 100.0 and chl_s > mit_s and chl_s > nt_s and c_len < 1000000:
                    note = 'chl'
                elif mit_s > 5000.0 and depth_value > 100.0 and chl_s < mit_s and mit_s > nt_s and c_len < 1000000:
                    note = 'mit'
                elif cont_b is True and depth_value < 10.0 and c_len < 1000000:
                    note = 'contamination'
                else:
                    note = 'nuc'

                output_list = [c_id,
                               str(c_len),
                               str(gc_value),
                               str(depth_value),
                               str(chl_s),
                               str(mit_s),
                               str(nt_s),
                               str(cont_b),
                               note
                               ]

                output_string = "\t".join(output_list)
                f.write(output_string+"\n")

    # plot
    output_plot_file = args.work_dir + "/ContigFilter.svg"
    contig_filter_plot(args.contig_filter_results, output_plot_file)
